package market;

import data.CatalogItem;
import data.CatalogItemValue;
import data.IntegerValue;
import data.QuoteValue;
import data.Value;
import data.ooimpl.CatalogItemImpl;

/**
 * A CatalogItemImpl that represents the articles of the market.
 */
public class CIArticle extends CatalogItemImpl {

    private String name;
    private String[] description = new String[] {"", ""};
    private String category;

    /**
     * @param id ID of the CIArticle.
     * @param name name of the CIArticle
     * @param category the CIArticle's category.
     * @param bid the price a customer has to pay for this CIArticle.
     * @param offer the cost of this CIArticle when the manager orders it.
     */
    public CIArticle(String id, String name, String category, int bid, int offer) {
        super(id, new QuoteValue(new IntegerValue(bid), new IntegerValue(offer)));
        this.name = name;
        this.category = category;
    }

    /**
     * @param id ID of the CIArticle.
     * @param name name of the CIArticle
     * @param category the CIArticle's category.
     * @param bid the price a customer has to pay for this CIArticle.
     * @param offer the cost of this CIArticle when the manager orders it.
     */
    public CIArticle(String id, String name, String category, IntegerValue bid, IntegerValue offer) {
        super(id, new QuoteValue(bid, offer));
        this.name = name;
        this.category = category;
    }

    /**
     * Sets the name of the article.
     *
     * @param name the article's name.
     */
    public void setArticleName(String name) {
        this.name = name;
    }

    /**
     * Set the description of the article.
     *
     * @param description the description of this article.
     */
    public void setDescription(String[] description) {
        this.description = description;
    }

    /**
     * @return the article's name.
     */
    public String getArticleName() {
        return name;
    }

    /**
     * @return the article's category.
     */
    public String getCategory() {
        return category;
    }

    /**
     * @return the description of this article.
     */
    public String[] getDescription() {
        return description;
    }

    /**
     * @return the article's offer.
     */
    public int getOffer() {
      return new Integer(((QuoteValue)getValue()).getOffer().toString()).intValue();
    }

    /**
     * @return the article's bid.
     */
    public int getBid() {
      return new Integer(((QuoteValue)getValue()).getBid().toString()).intValue();
    }

    /**
     * Sets the article's bid.
     * @param vBid the new bid.
     */
    public void setBid(Value vBid) {
        setValue(new QuoteValue(vBid, ((QuoteValue)getValue()).getOffer()));
    }

    /**
     * @return a CatalogItemValue which returns the bid of a CIArticle.
     */
    public static CatalogItemValue getCatalogItemValue(){
        return new CatalogItemValue(){
            public Value getValue(CatalogItem ci) {
                CIArticle article = (CIArticle)ci;
                return ((QuoteValue)ci.getValue()).getBid();
            }
        };
    }

    /**
     * @return an identical clone of the given CatalogItemImpl
     */
    protected CatalogItemImpl getShallowClone() {
        QuoteValue qv = (QuoteValue)this.getValue();
        return new CIArticle(this.getName(), this.getArticleName(),
                this.getCategory(), (IntegerValue)qv.getBid(), (IntegerValue)qv.getOffer());
    }

    /**
     * Specifies how to compare CIArticles.
     * @param o the object to be compared with this CIArticle.
     * @return an int representing the result of the comparison.
     */
    public int compareTo(Object o) {
        return name.compareTo(((CIArticle)o).getArticleName());
    }
}
