package market;

import data.CatalogItemValue;
import data.CountingStock;
import data.DoubleValue;
import data.IntegerValue;
import data.ooimpl.CatalogItemImpl;

/**
 * A purchase order placed by the manager, which has not yet arrived.
 */
public class CIOpenPurchaseOrders extends CatalogItemImpl {

    private CountingStock cs;
    private String date;
    private int nr;

    /**
     * @param date date of the order.
     * @param nr order number, this parameter is necessary to distinguish between
     * two or more orders placed on the same day.
     * @param cs CountingStock with {@link CIArticle CIArticles} that are ordered.
     */
    public CIOpenPurchaseOrders(String date, int nr, CountingStock cs) {
        super(date + " (" + new Integer(nr).toString() + ")");
        this.date = date;
        this.nr = nr;
        this.cs = cs;
        this.setValue(new IntegerValue(2 + new Double(4 * Math.random()).intValue()));
    }

    /**
     * @return the date of the order.
     */
     public String getDate() {
        return date;
     }

    /**
     * @return the order number.
     */
     public int getOrderNumber() {
        return nr;
     }

    /**
     * @return the {@link CountingStock} with the orders.
     */
    public CountingStock getOrders() {
        return cs;
    }

    /**
     * @return the value of the orders.
     */
    public Double getOrdersValue() {
        DoubleValue dv = new DoubleValue(0);
        cs.sumStock(null, CatalogItemValue.EVALUATE_OFFER, dv);
        return Conversions.valueToDouble(dv);
    }

    /**
     * @return the number of days the market still has to wait for the delivery to arrive.
     */
    public int getDaysTillArrival() {
        return new Integer(getValue().toString()).intValue();
    }

    /**
     * Decreases the number of days the market has to wait for a delivery to arrive.
     */
    public void decreaseDaysTillArrival(int i) {
        setValue(getValue().subtract(new IntegerValue(i)));
    }

    /**
     * @return a clone of the given CatalogItemImpl.
     */
    protected CatalogItemImpl getShallowClone() {
        return new CIOpenPurchaseOrders(date, nr, cs);
    }
}
