package market;

import java.util.Iterator;

import data.CatalogItem;
import data.CountingStock;
import data.events.VetoException;
import data.ooimpl.CatalogImpl;
import data.ooimpl.CountingStockImpl;

/**
 * A Catalog containing {@link CIOpenPurchaseOrders}.
 */
public class COpenPurchaseOrders extends CatalogImpl {

    /**
     * @param name the catalogs's name.
     */
    public COpenPurchaseOrders(String name) {
        super(name);
    }

    /**
     * Removes a CatalogItem from this Catalog.
     * @param ci the CatalogItem to be removed.
     * @return the removed CatalogItem.
     */
    public CIOpenPurchaseOrders remove(CatalogItem ci) {
        try {
            return (CIOpenPurchaseOrders)super.remove(ci, null);
        }
        catch (VetoException e) {
            System.err.println("Fehler beim Lschen aus COpenPurchaseOrders");
            e.printStackTrace();
            return null;
        }
    }

    /**
     * Gets a CatalogItem from this Catalog.
     * @param key the searched CatalogItem's key.
     * @return the searched CatalogItem, <code>null</code> if not found.
     */
    public CIOpenPurchaseOrders get(String key) {
        try {
            return (CIOpenPurchaseOrders)super.get(key, null, false);
        }
        catch (VetoException e) {
            System.err.println("Fehler beim Holen eines Elements aus COpenPurchaseOrders");
            e.printStackTrace();
            return null;
        }
    }

    /**
     * Decreases the number of days to wait for open purchase orders according to the days that passed.
     * All purchases where the days to wait have dropped to or below zero are summed up and returned.
     * @param i the number of passed days.
     * @return the accumulated orders that arrived.
     */
    public CountingStock subtractPassedDays(int i) {
        CountingStock arrived = new CountingStockImpl("arrived", SMarket.getArticleCatalog());
        Iterator it = keySet(null).iterator();
        while (it.hasNext()) {
            CIOpenPurchaseOrders next = get((String)it.next());
            fireEditingCatalogItem(next, null); //update remaining days in table view (if open)
            next.decreaseDaysTillArrival(i);
            if (next.getDaysTillArrival() <= 0) {
                remove(next);
                arrived.addStock(next.getOrders(), null, false);
            }
        }
        return arrived;
    }
}