package market;

import sale.Shop;
import data.DataBasket;
import data.events.VetoException;
import data.ooimpl.CountingStockImpl;

/**
 * A CountingStockImpl which is used to store orders of customers,
 * which can be set active or not
 */
public class CSOrder extends CountingStockImpl{

    /**
     * Shows whether this CSOrder is active or not
     */
    private boolean isActive = true;

    /**
     * The time since this CSOrder exist
     */
    private Long time;

    /**
     * @param s the name of the CSOrder
     * @param active if true the CSOrder will be active
     */
    public CSOrder(String s, boolean active) {
        super(s, SMarket.getArticleCatalog());
        Shop.getTheShop().addStock(this);
        isActive = active;
    }

    /**
     * Removes a number of items from this CSOrder
     *
     * @param sKey the name of the StockItem
     * @param nCount the number of StockItems, that will be removed
     * @param db the databasket related to this transaction
     */
    public void remove(String sKey, int nCount, DataBasket db){
        try {
            super.remove(sKey, nCount, db);
        } catch (VetoException e) {
            System.err.println(e.getMessage());
        }
    }

    /**
     * Removes all items with the specified key from this CSOrder
     *
     * @param sKey the name of the StockItem
     * @return the number of items that are removed
     */
    public int removeAll(String sKey){
        int i = this.countItems(sKey, null);
        remove(sKey, i, null);
        return i;
    }

    /**
     * Returns whether this CSOrder is active or not
     *
     * @return true if this CSOrder is active, otherwise false
     */
    public boolean isActive() {
        return isActive;
    }

    /**
     * Sets this CSOrder active or not
     *
     * @param active if true the CSOrder will be active
     */
    public void setActive(boolean active) {
        isActive = active;
    }

    /**
     * Returns the time since this CSOrder exist
     *
     * @return the time since this CSOrder exist in milliseconds
     */
    public Long getTime() {
        return time;
    }

    /**
     * Returns a new CSOrder with a combination of owner-key and time as its key
     *
     * @param customer the owner of the new CSOrder
     * @param active if true the CSOrder will be active
     * @return the new CSOrder
     */
    public static CSOrder create(String customer, boolean active){
        Long t = new Long(System.currentTimeMillis());
        CSOrder cso = new CSOrder(customer+t.toString(), active);
        cso.time = t;
        return cso;
    }
}
