package market;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.JDisplayDialog;
import sale.JDisplayFrame;
import sale.MenuSheet;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.stdforms.MsgForm;

/**
 * Provides a {@link JDisplayDialog} that shows a simple {@link MsgForm}.
 */
public class JDDShowMessage {

    /**
     * Creates a JDisplayDialog that shows a short message.
     *
     * @param message the message to be shown.
     * @param caption the caption of the MsgForm.
     */
    public static JDisplayDialog showMessageDialog(String message, String caption) {
        final JDisplayDialog jdd = new JDisplayDialog(SMarket.getTheMarket().getShopFrame());
        final FormSheet fs = new MsgForm(caption, message, false);
        fs.addContentCreator(new FormSheetContentCreator(){
            protected void createFormSheetContent(final FormSheet fs) {
                fs.getButton(FormSheet.BTNID_OK).setAction(new sale.Action(){
                    public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                        fs.cancel();
                    }
                });
            }
        });
        try {
            jdd.popUpFormSheet(fs);
        } catch (InterruptedException e) {
            System.err.println(e.getMessage());
        }
        return jdd;
    }

    /**
     * Creates a JDisplayDialog that shows a short message, the JDisplayDialog will pop up
     * in the center of the given FormSheet.
     * The SalesPoint of the given FormSheet will be suspended while the message is shown.
     *
     * @param parent FormSheet to which the position of the JDisplayDialog is set relative to.
     * @param message the message to be shown.
     * @param caption the caption of the MsgForm.
     */
    public static JDisplayDialog showMessageDialog(final FormSheet parent, String message, String caption){
        final SalesPoint sp = parent.getSalesPoint();
        final MenuSheet ms = sp.getDisplay().getMenuSheet();
        final JDisplayFrame jdf = (sale.JDisplayFrame)parent.getSalesPoint().getDisplay();
        final JDisplayDialog jdd = new JDisplayDialog(jdf);
        final FormSheet fs = new MsgForm(caption, message, false);
        //WindowListener that causes to give the dialog the focus whenever the appropriate
        //FormSheet got it.
        final WindowListener wl = new WindowAdapter() {
                public void windowActivated(WindowEvent e) {
                    jdd.setLocationRelativeTo(parent.getComponent());
                    jdd.toFront();
                }
            };
        //WindowListener that causes wl to be removed and the process to resume when jdd is closed.
        jdd.addWindowListener(new WindowAdapter(){
            public void windowClosed(WindowEvent e) {
                jdf.removeWindowListener(wl);
                parent.attach(sp);
                if (ms != null) {
                    ms.attach(sp);
                }
                if (parent.getProcess() != null) {
                    parent.getProcess().resume();
                }
            }
        });
        fs.addContentCreator(new FormSheetContentCreator(){
            protected void createFormSheetContent(final FormSheet fs) {
                fs.getButton(FormSheet.BTNID_OK).setAction(new sale.Action(){
                    public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                        fs.cancel();
                    }
                });
            }
        });
        try {
            jdd.popUpFormSheet(parent, fs);
            jdf.addWindowListener(wl);
            if (parent.getProcess() != null) {
                parent.getProcess().suspend();  //stop current SaleProcess
            }
            if (ms != null) {
                ms.detachSalesPoint();          //remove MenuSheet
            }
            parent.detachSalesPoint();          //remove FormSheet
        } catch (InterruptedException e) {
            System.err.println(e.getMessage());
        }
        return jdd;
    }
}
