package market;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * The time format used by the market. This is merely a GregorianCalendar with an overwritten
 * toString() method.
 */
public class MarketCalendar extends GregorianCalendar {

    public MarketCalendar() {
        super();
    }

    /**
     * @param year the new Calendar's year.
     * @param month the new Calendar's month.
     * @param date the new Calendar's date.
     */
    public MarketCalendar(int year, int month, int date) {
        super(year, month, date);
    }

    /**
     * Takes an arbitrary Calenar and creates a MarketCalendar from it.
     * @param gc the original Calendar.
     * @return a MarketCalendar representing the same date as the passed Calendar.
     */
    public static MarketCalendar create(Calendar gc) {
        return new MarketCalendar(gc.get(YEAR), gc.get(MONTH), gc.get(DATE));
    }

    /**
     * @return the Calendar as dd.MM.yyyy (e.g. 01.01.2000).
     */
    public String toString() {
        SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy");
        return sdf.format(getTime());
    }
}