package market;

import java.io.Serializable;

/**
 * Contains some variables which affect the computation of the discount and the dismissal compensation.
 */
public class Options implements Serializable {

    private int discountRange = 12;
    private double maxDiscount = 0.35;
    private int discountValue = 15000;
    private double fractionOfWages = 1;
    private double timeOfEmployment = 0.05;

    /**
     * @return the number of months which have influence on the discount.
     */
    public int getDiscountRange() {
        return discountRange;
    }

    /**
     * Sets the number of months which have influence on the discount.
     * @param range the number of months
     */
    public void setDiscountRange(int range) {
        discountRange = range;
    }

    /**
     * @return the maximal discount that can be allowed.
     */
    public double getMaxDiscount() {
        return maxDiscount;
    }

    /**
     * Sets the maximal discount that can be allowed.
     * @param max the maximal discount.
     */
    public void setMaxDiscount(double max) {
        maxDiscount = Conversions.round(max, 5);
    }

    /**
     * @return the money in Euros that are worth one percent discount.
     */
    public int getDiscountValue() {
        return discountValue;
    }

    /**
     * Sets the money in Euros that are worth one percent discount.
     * @param value the money in Euros.
     */
    public void setDiscountValue(int value) {
        discountValue = value;
    }

    /**
     * @return the influence the current wage has on the computation of the dismissal compensation.
     */
    public double getFractionOfWages() {
        return fractionOfWages;
    }

    /**
     * Sets the influence the current wage has on the computation of the dismissal compensation.
     * @param fraction the influence of the wage in percent.
     */
    public void setFractionOfWages(double fraction) {
        fractionOfWages = Conversions.round(fraction, 5);
    }

    /**
     * @return the influence the time the worker has been employed has on the computation of the
     * dismissal compensation.
     */
    public double getTimeOfEmployment() {
        return timeOfEmployment;
    }

    /**
     * Sets the influence the time the worker has been employed has on the computation of the dismissal
     * compensation.
     * @param fraction the influence of the time in percent.
     */
    public void setTimeOfEmployment(double fraction) {
        timeOfEmployment = Conversions.round(fraction, 5);
    }
}