package market;

import market.event.MarketEventListener;
import market.resource.IconSetter;
import market.stdform.FSEmpty;
import market.stdform.FSWorkerDefault;
import sale.FormSheet;
import sale.JDisplayFrame;
import sale.SalesPoint;
import sale.events.FormSheetEvent;
import data.ooimpl.DataBasketImpl;

/**
 * A SalesPoint that can react to MarketEvents.
 */
public class SPListenable extends SalesPoint implements MarketEventListener {

    /**
     * @param s the SalesPoints name.
     */
    public SPListenable(String s) {
        super(s);
        attach(new DataBasketImpl());
    }

    /**
     * @return the default FormSheet.
     */
    protected FormSheet getDefaultFormSheet() {
        return new FSEmpty();
    }

    /**
     * Sets the image icon when a SalesPoint is opened.
     * @param e the fired Event
     */
    public void formSheetSet(FormSheetEvent e) {
        super.formSheetSet(e);
        IconSetter.setIcon((JDisplayFrame)getDisplay());
    }

    /**
     * Sets SalesPoint's icon after it is loaded from persistence file.
     */
    public void resume() {
        super.resume();
        IconSetter.setIcon((JDisplayFrame)getDisplay());
    }

    /**
     * @return <code>true</code> if no SaleProcess is running, otherwise <code>false</code>.
     * This forces the user to quit the SaleProcess before he closes the SalesPoint.
     */
    protected boolean onCanQuit() {
        return getCurrentProcess() == null;
    }

    /**
     * Reaction on event: The market is about to close.
     */
    public void notifyOnMarketClosing() {
        getDisplay().getFormSheet().setCaption(SMarket.MARKET_CLOSES_SHORT);
    }

    /**
     * Reaction en event: The market isn't about to close anymore.
     */
    public void notifyOnMarketNotClosing() {
        getDisplay().getFormSheet().setCaption(SMarket.MARKET_CLOSES_NOT);
    }

    /**
     * Reaction on event: The market has just closed.
     */
    public void marketClosed() {
        getDisplay().getFormSheet().setCaption(SMarket.MARKET_CLOSED);
    }

    /**
     * Reaction on event: The market has just opened.
     */
    public void marketOpened() {
        getDisplay().getFormSheet().setCaption(SMarket.MARKET_OPENED);
    }

    /**
     * Reaction on event: The time has advanced.
     */
    public void timeAdvanced() {
    }

    /**
     * Reaction on event: A new order for workers arrived or a worker logged on.
     */
    public void workerInformationChanged(){
        if(this.getCurrentProcess() instanceof SProcessWorker){
            if(this.getDisplay().getFormSheet() instanceof FSWorkerDefault){
                ((SProcessWorker)this.getCurrentProcess()).getInitialGate();
            }
        }
    }
}