package market;

import market.stdform.ButtonIDs;
import market.stdform.FSCustomerConfirmSelection;
import market.stdform.FSCustomerOfferTable;
import market.stdform.MSLogOff;
import sale.FormSheet;
import sale.Gate;
import sale.GateChangeTransition;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Transition;
import sale.UIGate;
import sale.stdforms.MsgForm;
import users.User;
import data.CountingStock;
import data.IntegerValue;
import data.Value;

/**
 * The customer process. This process handles a customer's purchase.
 */
public class SProcessCustomer extends SProcessMarket{

    /**
     * The customer who interacts with this SaleProcess.
     */
    private UCustomer uc_customer;

    /**
     * Gate for selecting items to purchase.
     */
    private UIGate uig_selection = new UIGate(null, null);

    /**
     * Gate for affirming or canceling the purchase.
     */
    private UIGate uig_commit = new UIGate(null, null);

    /**
     * Gate for displaying the successful purchase.
     */
    private UIGate uig_ok = new UIGate(null, null);

    /**
     * @param user the customer who performs the purchase.
     */
    public SProcessCustomer(User user){
        super("buy");
        uc_customer = (UCustomer)(user);
    }

    // ################################## Gates ##########################################################

    /**
     * Attaches {@link FSCustomerOfferTable}, its actions and the menu to {@link #uig_selection}.
     * @return the set up {@link #uig_selection}.
     */
    protected Gate getInitialGate() {

        final FormSheet fs = FSCustomerOfferTable.getOfferFormSheet((CountingStock)SMarket.getOffer(),
                uc_customer.getShoppingBasket(),
                this.getBasket(),
                uig_selection);
        setAction(fs, new sale.Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                if(uc_customer.getShoppingBasket().size(getBasket())==0){
                    JDDShowMessage.showMessageDialog(fs, "Bitte whlen sie Artikel zum Kauf aus.",
                            "Keine Auswahl getroffen");
                }
                else uig_selection.setNextTransition(GateChangeTransition.CHANGE_TO_COMMIT_GATE);
            }
        }, ButtonIDs.BTN_BUY);
        setTransition(fs, GateChangeTransition.CHANGE_TO_ROLLBACK_GATE, ButtonIDs.BTN_BACK);
        uig_selection.setFormSheet(fs);
        uig_selection.setMenuSheet(new MSLogOff());
        return uig_selection;
    }

    /**
     * Attaches {@link FSCustomerConfirmSelection}, its actions and the menu to {@link #uig_commit}.
     * @return the set up {@link #uig_commit}.
     */
    public Gate getCommitGate() {
        FormSheet fs = new FSCustomerConfirmSelection(getAmount());
        setTransition(fs, commit(), ButtonIDs.BTN_BUY);
        setTransition(fs, new GateChangeTransition(getInitialGate()), ButtonIDs.BTN_BACK);
        uig_commit.setFormSheet(fs);
        return uig_commit;
    }

    /**
     * Attaches a {@link MsgForm} and its OK-action to {@link #uig_ok}.
     * @return the set up {@link #uig_ok}.
     */
    public Gate getOkGate() {
        FormSheet fs = new MsgForm("Auswahl besttigt!", "Bitte begeben Sie sich zur Kasse!");
        setTransition(fs, changeToStopGate(), FormSheet.BTNID_OK);
        uig_ok.setFormSheet(fs);
        return uig_ok;
    }


    // ############################ Transitions ###########################################################

    /**
     * @return a Transition that changes to the {@link #getStopGate()} and adds the customer to the global till-queue.
     */
    private Transition changeToStopGate(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                SICustomer.addToTillQueue(uc_customer);
                return getStopGate();
            }
        };
    }

    /**
     * @return a transition that commits the databasket and changes to the {@link #getOkGate()}.
     */
    private Transition commit(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                pOwner.getBasket().commit();
                return getOkGate();
            }
        };
    }

    //################################# private methods ########################################################

    /**
     * @return the Value of the articles in the customers shoppingbasket.
     */
    private Value getAmount(){
        return uc_customer.getShoppingBasket().sumStock(this.getBasket(), CIArticle.getCatalogItemValue(), new IntegerValue(0));
    }
}
