package market;

import java.awt.Rectangle;

import market.stdform.ButtonIDs;
import market.stdform.FSCheckable;
import market.stdform.FSEditPersonData;
import market.stdform.FSEmpty;
import sale.Action;
import sale.FormSheet;
import sale.Gate;
import sale.GateChangeTransition;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Shop;
import sale.Transition;
import sale.UIGate;
import sale.stdforms.MsgForm;
import users.User;

/**
 * SaleProcess that handles the new registration of a customer or
 * the editing of personal data of an existing customer.
 */
public class SProcessCustomerEditProfile extends SProcessMarket{

    /**
     * Gate for editing profile.
     */
    private UIGate uig_editProfile = new UIGate(null, null);

    /**
     * Gate for displaying the successful change of data.
     */
    private UIGate uig_confirmation = new UIGate(null, null);

    /**
     * Gate for displaying the change of data was unsuccessful.
     */
    private UIGate uig_passwordFailed = new UIGate(null, null);

    /**
     * The customer who interacts with this SaleProcess.
     */
    private UCustomer customer;

    private FSCheckable fsc_profile;

    /**
     * Shows whether it is the first registration
     * or only the correction of existing data.
     */
    private boolean firstRegistration;

    /**
     * @param user the UCustomer who interacts with this process.
     */
    public SProcessCustomerEditProfile(User user){
        super("Edit-Profile");
        customer = (UCustomer)user;
        if(user == null) firstRegistration = true;
        else firstRegistration = false;
    }


    // ################################## Gates ##########################################################

    /**
     * Attaches {@link FSEditPersonData}, its actions and the menu to {@link #uig_editProfile}.
     * @return the set up {@link #uig_editProfile}.
     */
    protected Gate getInitialGate() {
        fsc_profile = FSEditPersonData.getCustomerProfileForCustomer(customer);
        setAction(fsc_profile, new sale.Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                if(fsc_profile.checkTextFields(FSCheckable.ALL_ERRORMESSAGES_AT_ONCE, true)){
                    User usr = null;
                    if (customer == null) {
                        String userID = fsc_profile.getEntry(FSEditPersonData.JTFC_LOGIN);
                        usr = UMUserBase.getGlobalBase().getUser(userID);
                    }
                    if (usr != null) {
                        JDDShowMessage.showMessageDialog(fsc_profile, "Dieses Login wurde bereits " +
                                "vergeben, whlen sie bitte ein anderes.", "Doppeltes Login!");
                    }
                    else {
                        uig_editProfile.setNextTransition(commitEdits());
                    }
                }
            }
        }, ButtonIDs.BTN_ACCEPT);
        setAction(fsc_profile, new sale.Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                uig_editProfile.setNextTransition(GateChangeTransition.CHANGE_TO_STOP_GATE);
                if(firstRegistration){
                    if(sp.getCurrentProcess()!=null) sp.processFinished(sp.getCurrentProcess());
                    SMarket.getTheMarket().removeSalesPoint(sp);
                }
            }
        }, ButtonIDs.BTN_BACK);
        uig_editProfile.setFormSheet(fsc_profile);
        return uig_editProfile;
    }

    /**
     * Attaches a {@link MsgForm} and its OK-action to {@link #uig_confirmation}.
     * @return the set up {@link #uig_confirmation}.
     */
    private Gate getConfirmationGate(){
        FormSheet fs = new MsgForm("Daten gespeichert", "Ihre Daten wurden erfolgreich gendert!");
        setAction(fs, new sale.Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                uig_confirmation.setNextTransition(GateChangeTransition.CHANGE_TO_STOP_GATE);
                if(firstRegistration){
                    if(sp.getCurrentProcess()!=null) sp.processFinished(sp.getCurrentProcess());
                    SMarket.getTheMarket().removeSalesPoint(sp);
                }
            }
        }, FormSheet.BTNID_OK);
        uig_confirmation.setFormSheet(fs);
        return uig_confirmation;
    }

    /**
     * Attaches a {@link MsgForm} and its OK-action to {@link #uig_passwordFailed}.
     * @return the set up {@link #uig_passwordFailed}.
     */
    private Gate getPasswordFailedGate(){
        FormSheet fs = new MsgForm("Passwort berprfen", "Sie haben entweder kein Passwort eingegeben\n"+
        "oder die Passwrter stimmen nicht berein");
        setTransition(fs, new GateChangeTransition(uig_editProfile), FormSheet.BTNID_OK);
        uig_passwordFailed.setFormSheet(fs);
        return uig_passwordFailed;
    }


    // ############################ Transitions ###########################################################

    /**
     * @return a transition that changes to the {@link #getConfirmationGate()} if all new passwords are equal,
     * otherwise to the {@link #getPasswordFailedGate()}.
     */
    private Transition commitEdits(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                FSEditPersonData fs = ((FSEditPersonData)fsc_profile.getFormSheet());
                if(fs.passwordsEqual()){
                    if(firstRegistration){
                        if(fs.getPassword()==null) return getPasswordFailedGate();
                        String userID = fsc_profile.getEntry(FSEditPersonData.JTFC_LOGIN);
                        customer = (UCustomer)UMUserBase.createUser(userID, UMUserBase.CUSTOMER, null);
                    }
                    customer.setSalutation(fs.getSalutation());
                    customer.setFirstName(fsc_profile.getEntry(FSEditPersonData.JTFC_FIRSTNAME));
                    customer.setSurname(fsc_profile.getEntry(FSEditPersonData.JTFC_NAME));
                    customer.setTelephone(fsc_profile.getEntry(FSEditPersonData.JTFC_TELEPHONE));
                    customer.setStreet(fsc_profile.getEntry(FSEditPersonData.JTFC_STREET));
                    customer.setPostcode(Integer.parseInt(fsc_profile.getEntry(FSEditPersonData.JTFC_POSTCODE)));
                    customer.setCity(fsc_profile.getEntry(FSEditPersonData.JTFC_CITY));
                    customer.setCompany(fsc_profile.getEntry(FSEditPersonData.JTFC_COMPANY));
                    if(fs.getPassword()!=null) customer.setPassWd(User.garblePassWD(fs.getPassword()));
                    return getConfirmationGate();
                 }
                 return getPasswordFailedGate();
            }
        };
    }


    //################################### public methods #######################################################

    /**
     *@return an action that creates a new {@link SPListenable} and runs a new SProcessCustomerEditProfile on it
     */
    public static Action create(){
        return new sale.Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                SPListenable point = new SPListenable("Sohn&Sohn"){
                    protected FormSheet getDefaultFormSheet() {
                        return new FSEmpty();
                    }
                    // Quit the SalesPoint without any question
                    /*protected boolean onCanQuit(){
                        return getCurrentProcess() == null;
                    }*/
                };
                point.setSalesPointFrameBounds(new Rectangle(0,0,640,540));
                Shop.getTheShop().addSalesPoint(point);
                point.runProcess(new  SProcessCustomerEditProfile(null));
            }
        };
    }
}
