package market;

import java.awt.Rectangle;

import market.stdform.FSLogOn;
import sale.Action;
import sale.FormSheet;
import sale.Gate;
import sale.GateChangeTransition;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Shop;
import sale.UIGate;
import sale.stdforms.MsgForm;
import users.User;
import users.UserManager;
import users.stdforms.LogOnForm;
import users.swing.UserFilter;

/**
 * SaleProcess used to log on a UPerson to the system.
 */
public class SProcessLogOn extends SProcessMarket {

    /**
     * Stores which kind of person wants to log on, look also at constants in UMUserBase
     */
    private int userType;

    /**
     * UserFilter used to show the right Users in a LogOnForm
     */
    private UserFilter uf_filter;

    /**
     * LogOnForm used by select-user-gate
     */
    private LogOnForm lof_selection;

    /**
     * Gate for selecting username.
     */
    private UIGate uig_selectUser = new UIGate(null, null);

    /**
     * Gate for displaying the log on was unsuccessful.
     */
    private UIGate uig_logOnFailed = new UIGate(null, null);

    /**
     * Gate for displaying the user has already logged on.
     */
    private UIGate uig_userIsLogged = new UIGate(null, null);

    /**
     * Create a new SProcessLogOn
     *
     * @param i the kind of User who wants to log on
     */
    private SProcessLogOn(int i){
        super("Log-on-Process");
        this.userType = i;

        // choice what kind of user wants to log on
        switch(userType){
            case UMUserBase.CUSTOMER:
                uf_filter = ((UMUserBase)UserManager.getGlobalUM()).getCustomers();
                break;
            case UMUserBase.WAREHOUSE_WORKER:
                uf_filter = ((UMUserBase)UserManager.getGlobalUM()).getWarehouseWorker();
                break;
            case UMUserBase.SELLER:
                uf_filter = ((UMUserBase)UserManager.getGlobalUM()).getSeller();
                break;
            case UMUserBase.MANAGER:
                uf_filter = ((UMUserBase)UserManager.getGlobalUM()).getManager();
        }
    }

    /**
     * Attaches a {@link FSLogOn} and its actions to {@link #uig_selectUser}.
     * @return the set up {@link #uig_selectUser}.
     */
    protected Gate getInitialGate() {
        lof_selection = new FSLogOn(uf_filter);
        setAction(lof_selection, new Action(){
            public void doAction(SaleProcess process, SalesPoint point) {
                int error = 0;
                lof_selection.ok();
                User user = lof_selection.getResult();
                if(UMUserBase.isLoggedOn(user)) error = 1;
                if(user == null) error = 2;
                switch (error) {
                    case 1:
                    	uig_selectUser.setNextTransition(
                                new GateChangeTransition(getUserIsLoggedGate()));
                        break;
                    case 2:
                    	uig_selectUser.setNextTransition(
                                new GateChangeTransition(getLogOnFailedGate()));
                        break;
                    default:
                        process.detachContext();
                        switch(userType){
                            case UMUserBase.CUSTOMER:
                                SPCustomer sp = new SPCustomer(user);
                                SMarket.getTheMarket().removeSalesPoint(point);
                                uig_selectUser.setNextTransition(GateChangeTransition.CHANGE_TO_STOP_GATE);
                                break;
                            case UMUserBase.WAREHOUSE_WORKER:
                                point.attach(user);
                                point.setSalesPointFrameBounds(new Rectangle(0,0,640,480));
                                SMarket.fireUpdateWorkerScreen();
                                point.runProcess(new SProcessWorker(((UStaffer)user).getFullName()));
                                uig_selectUser.setNextTransition(GateChangeTransition.CHANGE_TO_STOP_GATE);
                                break;
                            case UMUserBase.SELLER:
                                point.attach(user);
                                point.setSalesPointFrameBounds(new Rectangle(0,0,640,480));
                                point.runProcess(new SProcessSeller(((UStaffer)user).getFullName()));
                                uig_selectUser.setNextTransition(GateChangeTransition.CHANGE_TO_STOP_GATE);
                                break;
                            case UMUserBase.MANAGER:
                                point.attach(user);
                                point.setSalesPointFrameBounds(new java.awt.Rectangle(0,0,720,540));
                                point.runProcess(new SProcessManager());
                                uig_selectUser.setNextTransition(GateChangeTransition.CHANGE_TO_STOP_GATE);
                                break;
                            }
                        break;
                }
            }}, FormSheet.BTNID_OK);

        setAction(lof_selection, new Action(){
            public void doAction(SaleProcess process, SalesPoint point) throws Throwable {
                uig_selectUser.setNextTransition(GateChangeTransition.CHANGE_TO_STOP_GATE);
                point.processFinished(process);
                SMarket.getTheMarket().removeSalesPoint(point);
            }
        }, FormSheet.BTNID_CANCEL);

        uig_selectUser.setFormSheet(lof_selection);
        return uig_selectUser;
    }

    /**
     * Attaches a {@link MsgForm} and its OK-action to {@link #uig_logOnFailed}.
     * @return the set up {@link #uig_logOnFailed}.
     */
    private Gate getLogOnFailedGate(){
        FormSheet fs = new MsgForm("Anmeldung fehlgeschlagen", "Das System konnte Sie nicht anmelden!\n"
                + "Bitte berprfen Sie Nutzernamen und Passwort.");
        setTransition(fs, new GateChangeTransition(getInitialGate()), FormSheet.BTNID_OK);
        uig_logOnFailed.setFormSheet(fs);
        return uig_logOnFailed;
    }

    /**
     * Attaches a {@link MsgForm} and its OK-action to {@link #uig_userIsLogged}.
     * @return the set up {@link #uig_userIsLogged}.
     */
    private Gate getUserIsLoggedGate(){
        FormSheet fs = new MsgForm("Anmeldung fehlgeschlagen", "Sie sind bereits am System angemeldet!");
        setTransition(fs, new GateChangeTransition(getInitialGate()), FormSheet.BTNID_OK);
        uig_userIsLogged.setFormSheet(fs);
        return uig_userIsLogged;
    }


    // ################################### public methods ##################################################

    /**
     * Returns an Action that initiates a SProcessLogOn
     *
     * @param i what kind of user wants to log on, look at the constants of UMUserBase
     *
     * @see UMUserBase
     */
    public static Action createLogOnProcess(final int i){
        return new Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                SProcessLogOn splo = new SProcessLogOn(i);
                SPListenable point = new SPListenable("Sohn&Sohn");
                 point.setSalesPointFrameBounds(new Rectangle(0,0,500,125));
                 Shop.getTheShop().addSalesPoint(point);
                 point.runProcess(splo);
            }
        };
    }
}

