package market;

import market.stdform.ButtonIDs;
import market.stdform.FSCheckable;
import market.stdform.FSSellerBill;
import market.stdform.FSSellerCustomerTable;
import market.stdform.FSSellerOrderTable;
import market.stdform.MSLogOff;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.Gate;
import sale.GateChangeTransition;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Transition;
import sale.UIGate;
import sale.stdforms.MsgForm;
import users.User;
import data.IntegerValue;
import data.Value;
import data.events.VetoException;
import data.stdforms.SingleTableFormSheet;


/**
 * The seller process. This process handles the payment of the orders.
 */
public class SProcessSeller extends SProcessMarket{

    /**
     * Gate for selecting customers.
     */
    private UIGate uig_customerSelection = new UIGate(null, null);

    /**
     * Gate for affirming the cancel of the order of the currently selected customer
     * and for displaying lost articles in case of shortages.
     */
    private UIGate uig_cancelBuy = new UIGate(null, null);

    /**
     * Gate for affirming the order of the currently selected customer.
     */
    private UIGate uig_order = new UIGate(null, null);

    /**
     * Gate for selecting discount and mode of payment.
     */
    private UIGate uig_commit = new UIGate(null, null);

    /**
     * Gate for displaying that no customer is selected.
     */
    private UIGate uig_noCustomer = new UIGate(null, null);

    /**
     * Gate for displaying that the payment was successful.
     */
    private UIGate uig_log = new UIGate(null, null);

    /**
     * Table that shows the queue of customers
     */
    private SingleTableFormSheet stfs_customerQueue;

    /**
     * FormSheet that shows the bill
     */
    private FSCheckable fs_sellerBill;

    /**
     * The UCustomer associated to the currently selected SICustomer
     */
    private UCustomer uc_customer;

    /**
     * The currently selected SICustomer
     */
    private SICustomer sic_customer;


    /**
     * @param name the name of the process.
     */
    public SProcessSeller(String name){
        super(name);
    }


    // ############################ Gates #####################################

    /**
     * Attaches {@link FSSellerCustomerTable}, its actions and the menu to {@link #uig_customerSelection}.
     * @return the set up {@link #uig_customerSelection}.
     */
    protected Gate getInitialGate() {
        stfs_customerQueue = FSSellerCustomerTable.getCustomerTable(SMarket.getTillQueue());
        stfs_customerQueue.setGate(uig_customerSelection);

        setTransition(stfs_customerQueue, changeToOrderGate(), ButtonIDs.BTN_OK);
        setTransition(stfs_customerQueue, changeToCancelBuyGate(), ButtonIDs.BTN_CANCEL);

        uig_customerSelection.setFormSheet(stfs_customerQueue);
        uig_customerSelection.setMenuSheet(new MSLogOff());
        return uig_customerSelection;
    }

    /**
     * Attaches a {@link MsgForm} and its ok-action to {@link #uig_cancelBuy}.
     * @return the set up {@link #uig_cancelBuy}.
     */
    protected Gate getCancelBuyGate(){
        FormSheet fs = new MsgForm("Auftrag stornieren?",
                "Sind Sie sicher, dass sie den Auftrag von "
                + uc_customer.getFullName() +
                " stornieren wollen?");

        fs.addContentCreator(new FormSheetContentCreator(){
            protected void createFormSheetContent(FormSheet fs) {
                 fs.getButton(FormSheet.BTNID_OK).setAction(new sale.Action(){
                    public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                        uig_cancelBuy.setNextTransition(cancelBuy());
                    }
                });
                fs.addButton("Zurck", FormSheet.BTNID_CANCEL, new sale.Action(){
                    public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                        uig_cancelBuy.setNextTransition(GateChangeTransition.CHANGE_TO_ROLLBACK_GATE);
                    }
                });
            }
        });

        uig_cancelBuy.setFormSheet(fs);
        return uig_cancelBuy;
    }

    /**
     * Attaches a {@link MsgForm} and its ok-action to {@link #uig_noCustomer}.
     * @return the set up {@link #uig_noCustomer}.
     */
    protected Gate getNoCustomerGate(){
        FormSheet fs = new MsgForm("Kein Kunde selektiert", "Sie mssen zunchst einen Kunden auswhlen!");
        setTransition(fs, GateChangeTransition.CHANGE_TO_ROLLBACK_GATE, FormSheet.BTNID_OK);
        uig_noCustomer.setFormSheet(fs);
        return uig_noCustomer;
    }

    /**
     * Attaches {@link FSSellerOrderTable} and its actions to {@link #uig_order}.
     * @return the set up {@link #uig_order}.
     */
    protected Gate getOrderGate(){
        FormSheet fs = FSSellerOrderTable.getOrderTable(uc_customer.getShoppingBasket(),
            sic_customer);
        setTransition(fs, GateChangeTransition.CHANGE_TO_COMMIT_GATE, ButtonIDs.BTN_ACCEPT);
        setTransition(fs, GateChangeTransition.CHANGE_TO_ROLLBACK_GATE, ButtonIDs.BTN_BACK);
        uig_order.setFormSheet(fs);
        return uig_order;
    }

    /**
     * Attaches {@link FSSellerBill} and its actions to {@link #uig_commit}.
     * @return the set up {@link #uig_commit}.
     */
    public Gate getCommitGate() {
        fs_sellerBill = FSSellerBill.create(uc_customer.getShoppingBasket().sumStock(
                    null,
                    CIArticle.getCatalogItemValue(),
                    new IntegerValue(0)),
                    uc_customer.getDiscount());

        setAction(fs_sellerBill, new sale.Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                if(fs_sellerBill.checkTextFields(FSCheckable.ALL_ERRORMESSAGES_AT_ONCE, false)){
                    uig_commit.setNextTransition(changeToConfirmationGate());
                }
            }
        }, ButtonIDs.BTN_ACCEPT);
        setTransition(fs_sellerBill, new GateChangeTransition(getOrderGate()), ButtonIDs.BTN_BACK);

        uig_commit.setFormSheet(fs_sellerBill);
        return uig_commit;
    }

    /**
     * @return the Gate to jump to if the selected customer has to be rolled back.
     */
    public Gate getRollbackGate() {
        return new Gate(){
            public Transition getNextTransition(SaleProcess pOwner, User usr)
                    throws InterruptedException {
                return new Transition(){
                    public Gate perform(SaleProcess pOwner, User usr) {
                        pOwner.getBasket().rollback();
                        uc_customer = null;
                        sic_customer = null;
                        return getInitialGate();
                    }
                };
            }
        };
    }

    /**
     * Attaches {@link MsgForm} and its ok-action to {@link #uig_log}.
     * @return the set up {@link #uig_log}.
     */
    public Gate getLogGate() {
        FormSheet fs = new MsgForm("Bezahlung erfolgt!",
                    "Der Kaufvorgang wurde abgeschlossen, die Artikel werden so schnell wie mglich ausgeliefert!");
        setTransition(fs, changeToCustomerSelectionGate(), FormSheet.BTNID_OK);
        uig_log.setFormSheet(fs);
        return uig_log;
    }

    // ####################### Transitions ##############################

    /**
     * @return a Transition that changes to the {@link #getInitialGate()} and logs the process.
     */
    private Transition changeToCustomerSelectionGate(){
        return new Transition(){
            public Gate perform(SaleProcess p, User usr) {
                try {
                  p.log (p);
                }
                catch (java.io.IOException ioe) {
                  throw new Error ("Exception occurred while logging process: " + ioe);
                }
                return getInitialGate();
            }
        };
    }

    /**
     * @return a Transition that changes to the {@link #getInitialGate()},
     * puts the articles of the selected customer back to the markets offer
     * and removes the customer permanently from the till-queue.
     */
    private Transition cancelBuy(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                SMarket.getOffer().addStock(uc_customer.getShoppingBasket(), null, true);
                pOwner.getBasket().commit();
                return getInitialGate();
            }
        };
    }

    /**
     * @return a Transition that changes to the {@link #getCancelBuyGate()}, if a customer is selected,
     * otherwise it will change to the {@link #getNoCustomerGate()}
     */
    private Transition changeToCancelBuyGate(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                if(stfs_customerQueue.getSelectedRecord()==null){
                    return getNoCustomerGate();
                }
                else{
                    setCustomer();
                    return getCancelBuyGate();
                }
            }
        };
    }

    /**
     * @return a Transition that changes to the {@link #getOrderGate()}, if a customer is selected,
     * otherwise it will change to the {@link #getNoCustomerGate()}
     */
    private Transition changeToOrderGate(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                if(stfs_customerQueue.getSelectedRecord()==null){
                    return getNoCustomerGate();
                }
                else{
                    setCustomer();
                    return getOrderGate();
                }
             }
        };
    }

    /**
     * @return a Transition that changes to {@link #getLogGate()},
     * adds the customers order to order-queue and
     * adds the payed money to markets account
     */
    private Transition changeToConfirmationGate(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                Value v = ((FSSellerBill)fs_sellerBill.getFormSheet()).getEndSum();
                    double discount = Conversions.round(
                            Double.valueOf(Conversions.convertComma(
                            fs_sellerBill.getEntry(FSSellerBill.JTFC_DISCOUNT))).doubleValue(), 3);
                //show statistics not until day-end closing, so store them im dailyStats
                SMarket.getDailySalesStats().addSales(uc_customer, discount);
                SMarket.getDailySalesStats().addRevenue(Conversions.valueToInt(v));
                SMarket.getCustomerStats().addSales(uc_customer, v);
                SMarket.addToAccount(v);
                if(uc_customer.getShoppingBasket().size(null)>0) SICustomer.addToOrderQueue(uc_customer);
                pOwner.getBasket().commit();
                uc_customer = null;
                sic_customer = null;
                return getLogGate();
            }
        };
    }


    // ########################### private methods #########################################

    /**
     * Sets customer from tableselection and remove it from customer-queue
     */
    private void setCustomer(){
        sic_customer = (SICustomer)stfs_customerQueue.getSelectedRecord();
        uc_customer = sic_customer.getCustomer();
        try {
            sic_customer.getStock().remove(sic_customer, this.getBasket());
        } catch (VetoException e) {
            System.err.println(e.getMessage());
        }
    }
}
