package market;

import market.statistics.Statistics;
import data.ooimpl.CountingStockImpl;

/**
 * This class implements the customers of the market.
 */
public class UCustomer extends UPerson {

    private String company;
    private CountingStockImpl csi_shoppingBasket;

    /**
     * Creates a new UCustomer.
     * @param userName the customer's ID.
     */
    public UCustomer(String userName){
        super(userName);
        csi_shoppingBasket = new CountingStockImpl(userName, SMarket.getArticleCatalog());
    }

    /**
     * Sets the customer's company.
     * @param company the company to be set.
     */
    public void setCompany(String company) {
        this.company = company;
    }

    /**
     * Gets the customer's company.
     * @return the company.
     */
    public String getCompany() {
        return company;
    }

    /**
     * Gets the customer's shopping basket.
     * @return the shopping basket.
     */
    public CountingStockImpl getShoppingBasket() {
        return csi_shoppingBasket;
    }

    /**
     * Computes and returns the customer's current discount.<br>
     * The discount is a value in the range from 0 to 1.
     * @return the discount.
     * @see Options
     */
    public double getDiscount() {
        int allowable = Statistics.getAllowableCustomerRevenue(this);
        int percentValue = SMarket.getOptions().getDiscountValue();
        //allowable (in cents) / 100 (Euro) / percentValue (how many percent worth) / 100 (range [0..1))
        double actualDiscount = new Integer(allowable).doubleValue()/(10000 * percentValue);
        return Math.min(actualDiscount, SMarket.getOptions().getMaxDiscount());
    }
}
