package market;

/* Generated by Together */
import java.util.Calendar;

import users.User;

/**
 * A person that interacts with the market. Can be a {@link UCustomer customer} or an
 * {@link UStaffer employee}.
 */
public class UPerson extends User {

    public static String HERR = new String("Herr");
    public static String FRAU = new String("Frau");

    private String salutation;
    private String name;
    private String firstName;
    private String street;
    private int postcode;
    private String city;
    private String telephone;
    private Calendar date;

    /**
     * Creates a new person.
     * @param userName the person's ID.
     */
    public UPerson(String userName){
        super(userName);
        date = (Calendar)SMarket.getTime().clone();
    }

    /**
     * Sets the person's name.
     * @param salutation the person's salutation (Mr, Mrs)
     * @param name the person's surname.
     * @param firstName the person's forename.
     */
    public void setFullName(String salutation, String name, String firstName) {
        this.salutation = salutation;
        this.name = name;
        this.firstName = firstName;
    }

    /**
     * Sets the person's address.
     */
    public void setAddress(String street, int postcode, String city) {
        this.street = street;
        this.postcode = postcode;
        this.city = city;
    }

    /**
     * Sets the person's telephone number.
     */
    public void setTelephone(String telephone) {
        this.telephone = telephone;
    }

    /**
     * Gets the person's name.
     * @return the full name, containing salutation, forename and surname.
     */
    public String getFullName() {
        return (salutation + " " + firstName + " " + name);
    }

    /**
     * Gets the person's address.
     * @return the address, containing street, postcode and city.
     */
    public String getAdress() {
        return (street + Integer.toString(postcode) + city);
    }

    /**
     * Gets the person's telephone number.
     * @return the telephone number.
     */
    public String getTelephone() {
        return telephone;
    }

    /**
     * Gets the person's salutation.
     * @return the salutation.
     */
    public String getSalutation() {
        return salutation;
    }

    /**
     * Sets the person's salutation.
     * @param salutation the salutation to be set.
     */
    public void setSalutation(String salutation) {
        this.salutation = salutation;
    }

    /**
     * Gets the person's surname.
     * @return the surname.
     */
    public String getSurname() {
        return name;
    }

    /**
     * Sets the person's surname.
     * @param name the surname to be set.
     */
    public void setSurname(String name) {
        this.name = name;
    }

    /**
     * Gets the person's forename.
     * @return the forename.
     */
    public String getFirstName() {
        return firstName;
    }

    /**
     * Sets the person's forename.
     * @param firstName the forename to be set.
     */
    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    /**
     * Gets the street where the person lives.
     * @return the street.
     */
    public String getStreet() {
        return street;
    }

    /**
     * Sets the street where the person lives.
     * @param street the street to be set.
     */
    public void setStreet(String street) {
        this.street = street;
    }

    /**
     * Gets the person's postcode.
     * @return the postcode.
     */
    public int getPostcode() {
        return postcode;
    }

    /**
     * Sets the person's postcode.
     * @param postcode the postcode to be set.
     */
    public void setPostcode(int postcode) {
        this.postcode = postcode;
    }

    /**
     * Gets the city where the person lives.
     * @return the city.
     */
    public String getCity() {
        return city;
    }

    /**
     * Sets the city where the person lives.
     * @param city the city.
     */
    public void setCity(String city) {
        this.city = city;
    }

    /**
     * Gets the day the person has first had contact with the market.<br>
     * For customer this is the registration, for workers the day of employment.
     *
     * @return the date registration or employment.
     */
    public Calendar getDayOfRegistration() {
        return date;
    }

    /**
     * Returns the time in months the person has been part of the market, be it as customer or worker.
     * @return the time of membership or employment.
     * @see #getDayOfRegistration
     */
    public int getMonthsOfMembership() {
        Calendar start = (Calendar)date.clone();
        Calendar now = SMarket.getTime();
        int months = 12 * (now.get(Calendar.YEAR) - start.get(Calendar.YEAR));
        months += now.get(Calendar.MONTH) - start.get(Calendar.MONTH);
        if (start.get(Calendar.DATE) > now.get(Calendar.DATE)) months--;
        return months;
    }

    /**
     * Sets the default comparison order for UPersons to <surname, forename>
     */
    public int compareTo(Object o) {
        if (o instanceof UPerson) {
            UPerson u = (UPerson) o;
            return new String(getSurname() + getFirstName()).compareTo(
                    new String(u.getSurname() + u.getFirstName()));
        } else {
            return 0;
        }
    }
}
