package market;

import data.IntegerValue;
import data.Value;

/**
 * A market's staffer. Can be warehouse worker, seller or manager.
 */
public class UStaffer extends UPerson {

    public static final String SELLER = new String("Kassierer");
    public static final String WAREHOUSE_WORKER = new String("Lagerarbeiter");
    public static final String MANAGER = new String("Manager");

    private IntegerValue ivSalary;
    private String qualification;

    /**
     * Creates a new UStaffer.
     * @param userName the staffer's ID.
     */
    public UStaffer(String userName, String qualification){
        super(userName);
        this.qualification = qualification;
    }

    /**
     * Sets the staffer's salary.
     * @param salary the salary to be set.
     */
    public void setSalary(int salary) {
        IntegerValue iv = new IntegerValue(salary);
        ivSalary = iv;
    }

    /**
     * Gets the staffer's salary.
     * @return the salary.
     */
    public IntegerValue getSalary() {
        return ivSalary;
    }

    /**
     * Gets the staffer's qualification.
     * @return the qualification.
     */
    public String getQualification(){
        return qualification;
    }

    /**
     * Sets the staffer's qualification.<br>
     * Can be warehouse worker, seller or manager.
     * @param s the qualification's denotation.
     */
    public void setQualification(String s) {
        qualification = s;
    }

    /**
     * Computes and returns the money a staffer gets, if he or she is dismissed.
     * @return the dismissal compensation.
     * @see Options
     */
    public Value computeDismissalCompensation() {
        Options o = SMarket.getOptions();
        return ivSalary.multiply(getMonthsOfMembership()*o.getTimeOfEmployment()*o.getFractionOfWages());
    }
}