package market;

/**
 * The simplest implementation of ValueChecker. It only checks a String for emptiness, if wanted.<br>
 * A VCDummy is used for text fields, that need to be a {@link market.swing.JTFCheckable} to allow
 * access from outside the defining class, but for which no content checking is required.
 */
public class VCDummy implements ValueChecker {

    /**
     * The identifier used by {@link #getErrorString()}.
     */
    protected String identifier;

    /**
     * Defines if an empty String is considered to be an error or not.
     */
    protected boolean mayBeEmpty;

    /**
     * @param identifier the identifier.
     * @param mayBeEmpty <code>true</code> if an empty should be considered valid, otherwise <code>false</code>.
     */
    public VCDummy(String identifier, boolean mayBeEmpty) {
        this.identifier = identifier;
        this.mayBeEmpty = mayBeEmpty;
    }

    /**
     * @param identifier the identifier.
     */
    public VCDummy(String identifier) {
        this.identifier = identifier;
        this.mayBeEmpty = false;
    }

    /**
     * @param content the String to be checked.
     * @return always <code>true</code> if {@link #mayBeEmpty} is <code>true</code>, otherwise it returns
     * <code>true</code> if <code>content</code> is not empty.
     */
    public boolean isValidValue(String content) {
        return (mayBeEmpty || !content.equals(""));
    }

    /**
     * @return an error message if the String is empty when it shouldn't.
     */
    public String getErrorString() {
        return "Fehlerhafte Eingabe im Feld " + identifier + ": Das Feld darf nicht leer sein.";
    }
}