package market.event;

/**
 * Listener interface that enables Objects to listen to market specific events and react on them.
 */
public interface MarketEventListener {

    /**
     * Invoked when the manager announces closing-time.
     */
    void notifyOnMarketClosing();

    /**
     * Invoked when the manager cancels closing-time announcement.
     */
    void notifyOnMarketNotClosing();

    /**
     * Invoked when the market opens.
     */
    void marketOpened();

    /**
     * Invoked when the market closes.
     */
    void marketClosed();

    /**
     * Invoked when the date changes.
     */
    void timeAdvanced();

    /**
     * Invoked when a job enters the {@link market.SMarket#ss_warehouseQueue warehouse queue} or a
     * warehouse worker logs on or off.
     */
    void workerInformationChanged();

}