package market.statistics;

import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;

import market.SMarket;
import data.ooimpl.CatalogItemImpl;

/**
 * Every customer has a dedicated statistics item, which is represented by this class.<br>
 * The statistics are saved as a list that contains date and value of every purchase the customer has
 * made. That makes it possible to get filtered statistics for a special range of time.
 */
public class CICustomerStats extends CatalogItemImpl {

    /**
     * Keeps track of a customer's purchases.
     */
    private List history = new LinkedList();

    /**
     * @param id the statistics entry's ID. It must be equal to the appropriate customer's ID.
     */
    public CICustomerStats(String id) {
        super(id);
    }

    /**
     * Adds a purchase to the {@link #history}.
     * @param value the value of the purchase.
     */
    public void add(int value) {
        history.add(new HistoryEntry((Calendar)SMarket.getTime().clone(), value));
    }

    /**
     * @return the {@link #history}.
     */
    public List getHistory() {
        return history;
    }

    public CatalogItemImpl getShallowClone() {
        CICustomerStats cics = new CICustomerStats(getName());
        cics.history = this.history;
        return cics;
    }
}