package market.statistics;

import java.io.Serializable;
import java.util.Calendar;

/**
 * Used by history lists to contain both a date and a value.
 * @see CISalesStats#orderHistory orderHistory
 * @see CISalesStats#priceHistory priceHistory
 * @see CICustomerStats#history customerHistory
 */
public class HistoryEntry implements Serializable {

    protected Calendar date;
    protected int value;

    /**
     * @param date the date of the entry.
     * @param value the value of the entry.
     */
    public HistoryEntry(Calendar date, int value) {
        this.date = (Calendar)date.clone(); //if date is not cloned, HistoryEntry's date will change whenever
        this.value = value;                 //Shop's date changes!
    }

    /**
     * @return the date of the entry.
     */
    public Calendar getDate() {
        return date;
    }

    /**
     * @return the value of the entry.
     */
    public int getValue() {
        return value;
    }

    public String toString() {
        return "Datum: " + date + "; Wert: " + value + ";";
    }

}