package market.statistics;

import java.util.Calendar;

import market.Conversions;
import market.SMarket;

public class PriceHistoryEntry extends HistoryEntry {

    private boolean changeInMorning;

    public PriceHistoryEntry(Calendar date, int value) {
        super(date, value);
        this.changeInMorning = !SMarket.isOpen() && SMarket.hasTimeAdvanced();
    }


    /**
     * Returns if an Entry is privsional or not. An entry is provisional (and can therefore be replaced) if
     * <ul>
     *   <li>it has been created in the morning before the market opened. The market hasn't opened and
     *          the date hasn't advanced since then, that means, it is still the same morning since the
     *          last price change.</li>
     *   <li>it has been created in the evening after the market closed, and this evening has not yet
     *          passed by, that means, it is still the same evening since the last price change.</li>
     *   <li>it has been created in the evening after the market closed, but now is the following morning and
     *          the market hasn't opened yet.</li>
     * </ul>
     *
     * @return if an Entry is provisional or not.
     */
    public boolean isProvisional() {
        return (((Conversions.dayDifference(date, SMarket.getTime()) == 0) &&
                changeInMorning && SMarket.hasTimeAdvanced()) //created this morning, still morning
                ||
                ((Conversions.dayDifference(SMarket.getTime(), date) == 1) &&
                !changeInMorning && !SMarket.hasTimeAdvanced()) //created this evening, still evening
                ||
                (Conversions.dayDifference(date, SMarket.getTime()) == 0 && //created yesterday, market not open
                !SMarket.isOpen() && SMarket.hasTimeAdvanced()));
    }
}


