package market.stdform;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Comparator;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import market.CIArticle;
import market.Conversions;
import market.swing.CmpNumbers;
import market.swing.JTADescriptionArea;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.SaleProcess;
import sale.UIGate;
import sale.stdforms.FormSheetStrategy;
import util.swing.AbstractTableEntryDescriptor;
import util.swing.TableEntryDescriptor;
import data.CountingStock;
import data.DataBasket;
import data.stdforms.TwoTableFormSheet;
import data.stdforms.twotableformsheet.CSCSStrategy;
import data.swing.CountingStockTableModel;

/**
 * This FormSheet displays a TwoTableFormSheet where customers can choose articles from the offer
 * and put them in there shopping-baskets, furthermore there is a description-field at which
 * a short description of the selected product is shown.
 */
public class FSCustomerOfferTable {

    /**
     * @return the offer to shopping-basket table.
     *
     * @param offer the markets offer.
     * @param shoppingBasket the shopping-basket of the customer.
     * @param db the DataBasket related to the transfers between offer and shopping-basket.
     * @param uig the UIGate at which this FormSheet is shown.
     */
    public static TwoTableFormSheet getOfferFormSheet(CountingStock offer, CountingStock shoppingBasket, DataBasket db, UIGate uig){
        CSCSStrategy cscss = new CSCSStrategy();
        cscss.setErrorHandler(new FormSheetStrategy.ErrorHandler(){
            public void error(SaleProcess p, int nErrorCode) {
                if(nErrorCode==FormSheetStrategy.ErrorHandler.NOT_ENOUGH_ELEMENTS_ERROR){
                    p.getCurrentGate();
                }
                else p.error(nErrorCode);
            }
        });
        final TwoTableFormSheet ttfs = TwoTableFormSheet.create(
                "Produktauswahl",
                offer,
                shoppingBasket,
                db,
                uig,
                null,
                null,
                false, false,
                getTED(),
                getTED(),
                cscss);

        ttfs.addContentCreator(new FormSheetContentCreator(){
            protected void createFormSheetContent(FormSheet fs) {
                JPanel jpMain = new JPanel();
                JPanel jpUpper = new JPanel();
                JPanel jpTables = new JPanel();
                JPanel jpDescription = new JPanel();
                JScrollPane jsc = new JScrollPane();
                final JTADescriptionArea da = new JTADescriptionArea();
                GridBagConstraints c = new GridBagConstraints();
                GridBagLayout gridbag = new GridBagLayout();

                ttfs.getLeftTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                    public void valueChanged(ListSelectionEvent e) {
                        if (!e.getValueIsAdjusting()&&(ttfs.getLeftSelectedRecord()!=null)) da.setDescription(Conversions.recordToCIArticle(ttfs.getLeftSelectedRecord()));
                    }
                });

                jpTables = (JPanel)fs.getComponent();
                jpUpper.setLayout(gridbag);
                      c.gridx = 0;
                      c.weightx = 1;
                      c.weighty = 1;
                      c.gridy = 0;
                      c.fill = GridBagConstraints.BOTH;
                    gridbag.setConstraints(jpTables, c);
                jpUpper.add(jpTables);
                jpDescription.setLayout(new BoxLayout(jpDescription, BoxLayout.X_AXIS));
                jpDescription.add(jsc);
                jsc.setViewportView(da);

                jpMain.setLayout(new BoxLayout(jpMain, BoxLayout.Y_AXIS));
                jpMain.add(jpUpper);
                jpMain.add(Box.createVerticalStrut(10));
                jpMain.add(jpDescription);
                fs.setComponent(jpMain);

                fs.removeAllButtons();
                fs.addButton("Kaufen", ButtonIDs.BTN_BUY, null);
                fs.addButton("Zurck", ButtonIDs.BTN_BACK, null);
            }
        });
        return ttfs;
    }

    /**
     * @return the TableEntryDescriptor of the table.
     */
    private static TableEntryDescriptor getTED(){
        return new AbstractTableEntryDescriptor(){
            public int getColumnCount() {
                return 4;
            }

            public String getColumnName(int nIdx) {
                return (new String[]{ "Artikel", "Kategorie", "Preis", "Anzahl"}) [nIdx];
            }

            public Class getColumnClass(int nIdx) {
                return (new Class[] {String.class, String.class, Number.class, Number.class}) [nIdx];
            }

            public Object getValueAt(Object oRecord, int nIdx) {
                CIArticle article = Conversions.recordToCIArticle(oRecord);
                int count = ((CountingStockTableModel.Record)oRecord).getCount();
                switch (nIdx) {
                    case 0: return article.getArticleName();
                    case 1: return article.getCategory();
                    case 2: return Conversions.valueToCurrency(CIArticle.getCatalogItemValue().getValue(article));
                    case 3: return new Integer(count).toString();
                }
                return null;
             }

             public boolean canSortByColumn(int nIndex) {
                return true;
             }

             public Comparator getColumnOrder(int nIndex) {
                switch (nIndex) {
                   case 1: return null ;
                   case 2: return new CmpNumbers(CmpNumbers.BID);
                   case 3: return new CmpNumbers(CmpNumbers.COUNT);
                   default: return null;
                }
             }
        };
    }
}
