package market.stdform;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import market.CIArticle;
import market.Conversions;
import market.SMarket;
import market.statistics.CISalesStats;
import market.statistics.EvaluateStatistics;
import market.swing.ComponentFactory;
import market.swing.HistoryEntryModel;
import sale.FormSheet;
import sale.FormSheetContentCreator;

/**
 * This FormSheet displays detailed statistics of a chosen article during a chosen range of time.
 */
public class FSManagerArticleStatsDetail extends FormSheet {

    /**
     * @param ciss the precomputed statistics to be displayed.
     */
    public FSManagerArticleStatsDetail(final CISalesStats ciss) {
        super("Artikelstatistik - Details", new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                final CIArticle cia = SMarket.getArticleCatalog().get(ciss.getArticleID());
                final EvaluateStatistics es = new EvaluateStatistics(ciss);
                JPanel jpMain = new JPanel();
                JPanel jpData = new JPanel();
                JPanel jpTables = new JPanel();
                JPanel jpLeftTable = new JPanel();
                JPanel jpRightTable = new JPanel();
                JPanel jpDataLabel1 = new JPanel();
                JPanel jpDataLabel2 = new JPanel();
                JPanel jpValueLabel1 = new JPanel();
                JPanel jpValueLabel2 = new JPanel();
                JPanel jpEdit = new JPanel();
                JPanel jpEditLabel1 = new JPanel();
                JPanel jpEditValue1 = new JPanel();
                JPanel jpEditCurrency1 = new JPanel();
                JScrollPane jsc1 = new JScrollPane();
                JScrollPane jsc2 = new JScrollPane();
                JTable jt1 = new JTable();
                JTable jt2 = new JTable();
                Dimension tableDimension = new Dimension(250, 150);
                GridBagConstraints c = new GridBagConstraints();
                GridBagLayout gridbag = new GridBagLayout();
                jpMain.setLayout(gridbag);
                    c.gridy = 0;
                    c.weightx = 1;
                    c.weighty = 0.1;
                    c.insets = new Insets(10,0,0,0);
                    c.anchor = GridBagConstraints.NORTH;
                  gridbag.setConstraints(jpData, c);
                    c.gridy = 1;
                    c.weightx = 1;
                    c.weighty = 0.9;
                    c.insets = new Insets(10,0,0,0);
                    c.anchor = GridBagConstraints.CENTER;
                  gridbag.setConstraints(jpTables, c);

                jpData.setLayout(new BoxLayout(jpData, BoxLayout.X_AXIS));
                jpData.setBorder(ComponentFactory.createInsetBorder("Statistik fr " + cia.getArticleName()));
                jpData.add(jpDataLabel1);
                    jpDataLabel1.setLayout(new GridLayout(4,1));
                    jpDataLabel1.add(new JLabel("Auf Lager:"));
                    jpDataLabel1.add(new JLabel("Einkaufspreis:"));
                    jpDataLabel1.add(new JLabel("Durchschn. Verkaufspreis:"));
                    jpDataLabel1.add(new JLabel("Durchschn. Gewinn:"));
                jpData.add(jpValueLabel1);
                    jpValueLabel1.setLayout(new GridLayout(4,1));
                    jpValueLabel1.add(ComponentFactory.createTextField(SMarket.getOffer().countItems(
                            ciss.getArticleID(), null), 7, false, ComponentFactory.RIGHT, false));
                    jpValueLabel1.add(ComponentFactory.createTextField(Conversions.doubleToCurrency(
                            cia.getOffer()), 7, false, ComponentFactory.RIGHT, false));
                    jpValueLabel1.add(ComponentFactory.createTextField(Conversions.doubleToCurrency(
                            es.getAveragePrice()), 7, false, ComponentFactory.RIGHT, false));
                    jpValueLabel1.add(ComponentFactory.createTextField(Conversions.doubleToCurrency(
                            es.getAveragePrice() - cia.getOffer()), 7, false, ComponentFactory.RIGHT, false));
                jpData.add(Box.createHorizontalStrut(25));
                jpData.add(jpDataLabel2);
                    jpDataLabel2.setLayout(new GridLayout(4,1));
                    jpDataLabel2.add(new JLabel("Gesamt verkauft:"));
                    jpDataLabel2.add(new JLabel("Durchschn. Verkauf pro Tag:"));
                    jpDataLabel2.add(new JLabel("Durchschn. Bestellmenge:"));
                    jpDataLabel2.add(new JLabel("Bestellabstand:"));
                jpData.add(jpValueLabel2);
                    jpValueLabel2.setLayout(new GridLayout(4,1));
                    jpValueLabel2.add(ComponentFactory.createTextField(es.getAmount(),
                            7, false, ComponentFactory.RIGHT, false));
                    jpValueLabel2.add(ComponentFactory.createTextField(
                            Conversions.round(es.getAverageItemsSold(), 3),
                            7, false, ComponentFactory.RIGHT, false));
                    jpValueLabel2.add(ComponentFactory.createTextField(
                            Conversions.round(es.getAverageOrderAmount(), 1),
                            7, false, ComponentFactory.RIGHT, false));
                    jpValueLabel2.add(ComponentFactory.createTextField(
                            Conversions.round(es.getAverageDaysBetweenOrders(), 1),
                            7, false, ComponentFactory.RIGHT, false));
                    jpTables.setLayout(new BoxLayout(jpTables, BoxLayout.X_AXIS));
                    jpTables.add(jpLeftTable);
                        jpLeftTable.setLayout(new BoxLayout(jpLeftTable, BoxLayout.Y_AXIS));
                        jpLeftTable.add(new JLabel("Preisverlauf:"));
                        jpLeftTable.add(Box.createVerticalStrut(10));
                        jpLeftTable.add(jsc1);
                            jsc1.setViewportView(jt1);
                                jt1.setPreferredScrollableViewportSize(tableDimension);
                                jt1.setModel(new HistoryEntryModel(es.getPriceHistory(),
                                        HistoryEntryModel.PRICE_HISTORY));
                    jpTables.add(Box.createHorizontalStrut(25));
                    jpTables.add(jpRightTable);
                        jpRightTable.setLayout(new BoxLayout(jpRightTable, BoxLayout.Y_AXIS));
                        jpRightTable.add(new JLabel("Nachbestellungen:"));
                        jpRightTable.add(Box.createVerticalStrut(10));
                        jpRightTable.add(jsc2);
                            jsc2.setViewportView(jt2);
                                jt2.setPreferredScrollableViewportSize(tableDimension);
                                jt2.setModel(new HistoryEntryModel(es.getOrderHistory(),
                                        HistoryEntryModel.ORDER_HISTORY));
                    jpMain.add(jpData);
                    jpMain.add(jpTables);
                fs.setComponent(jpMain);
                fs.removeAllButtons();
                fs.addButton("Zurck", ButtonIDs.BTN_BACK, null);
            }
        }, false);
    }
}
