package market.stdform;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import market.CFilter;
import market.CIArticle;
import market.Conversions;
import market.SMarket;
import market.swing.ComponentFactory;
import market.swing.JCTimeRangeBoxes;
import market.swing.JTADescriptionArea;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import util.swing.AbstractTableEntryDescriptor;
import util.swing.TableEntryDescriptor;
import data.Catalog;
import data.stdforms.SingleTableFormSheet;
import data.swing.CatalogTableModel;

/**
 * This FormSheet is used to choose the article and the range of time for which statistics
 * should be computed.
 */
public class FSManagerArticleStatsMain {

    private static JComboBox jcbFromMonth;
    private static JComboBox jcbToMonth;
    private static JComboBox jcbFromYear;
    private static JComboBox jcbToYear;
    private static JCTimeRangeBoxes jctrb = new JCTimeRangeBoxes();

    /**
     * Creates a {@link SingleTableFormSheet} with some extra components. The look of the table is
     * defined by the {@link TEDManagerArticleStatsMain}.
     *
     * @return the created SingleTableFormSheet.
     */
    public static SingleTableFormSheet create() {
        final TableEntryDescriptor ted = new TEDManagerArticleStatsMain();
        final Catalog source = SMarket.getArticleCatalog();
        final SingleTableFormSheet stfs = SingleTableFormSheet.create(
                "Artikelstatistik",
                source,
                null,                           //UIGate
                ted                             //TED
        );
        stfs.addContentCreator(new FormSheetContentCreator() {
            protected void createFormSheetContent(final FormSheet fs) {
                JPanel jpMain = new JPanel();
                JPanel jpTable = new JPanel();
                JPanel jpFilter = new JPanel();
                JPanel jpUpper = new JPanel();
                JPanel jpTimeRange = new JPanel();
                JPanel jpChooseArticle = new JPanel();
                JPanel jpFromSelection = new JPanel();
                JPanel jpFromDropdown = new JPanel();
                JPanel jpToSelection = new JPanel();
                JPanel jpToDropdown = new JPanel();
                JPanel jpDescription = new JPanel();
                jcbFromYear = jctrb.getFromYearBox();
                jcbToYear = jctrb.getToYearBox();
                jcbFromMonth = jctrb.getFromMonthBox();
                jcbToMonth = jctrb.getToMonthBox();
                JScrollPane jsc = new JScrollPane();
                GridBagConstraints c = new GridBagConstraints();
                GridBagLayout gridbag = new GridBagLayout();
                final JComboBox jcbFilter = new JComboBox(SMarket.getArticleCategories());
                jcbFilter.addItemListener(new ItemListener() {
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == e.SELECTED) {
                            int i = jcbFilter.getSelectedIndex();
                            CFilter cf = new CFilter(source, i); //filter CatalogItems
                            stfs.setTableModel(new CatalogTableModel(cf, null, null, ted));
                        }
                    }
                });
                final JTADescriptionArea da = new JTADescriptionArea();
                //add Listener to table that reacts on selection change
                stfs.getTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                    public void valueChanged(ListSelectionEvent e) {
                        if (!e.getValueIsAdjusting()) {
                            da.setDescription(Conversions.recordToCIArticle(stfs.getSelectedRecord()));
                        }
                    }
                });
                jpUpper.setLayout(gridbag);
                    c.gridx = 0;
                    c.weightx = 0;
                    c.weighty = 1;
                    c.gridy = 0;
                    c.fill = GridBagConstraints.BOTH;
                  gridbag.setConstraints(jpTimeRange, c);
                    c.gridx = 1;
                    c.weightx = 1;
                    c.weighty = 1;
                    c.gridy = 0;
                    c.fill = GridBagConstraints.BOTH;
                  gridbag.setConstraints(jpTable, c);
                    c.gridx = 2;
                    c.weightx = 0;
                    c.weighty = 1;
                    c.gridy = 0;
                    c.fill = GridBagConstraints.BOTH;
                  gridbag.setConstraints(jpFilter, c);
                jpUpper.add(jpTimeRange);
                    jpTimeRange.setLayout(new BoxLayout(jpTimeRange, BoxLayout.Y_AXIS));
                    jpTimeRange.add(jpFromSelection);
                        jpFromSelection.setBorder(ComponentFactory.createInsetBorder("von"));
                        jpFromSelection.add(jpFromDropdown);
                            jpFromDropdown.setLayout(new GridLayout(2,1));
                            jpFromDropdown.add(jcbFromMonth);
                            jpFromDropdown.add(jcbFromYear);
                    jpTimeRange.add(jpToSelection);
                        jpToSelection.setBorder(ComponentFactory.createInsetBorder("bis"));
                        jpToSelection.add(jpToDropdown);
                            jpToDropdown.setLayout(new GridLayout(2,1));
                            jpToDropdown.add(jcbToMonth);
                            jpToDropdown.add(jcbToYear);
                jpUpper.add(jpTable);
                    jpTable.setLayout(new BoxLayout(jpTable, BoxLayout.X_AXIS));
                    jpTable.add(fs.getComponent());
                jpUpper.add(jpFilter);
                    jpFilter.add(jcbFilter);
                jpDescription.setLayout(new BoxLayout(jpDescription, BoxLayout.X_AXIS));
                jpDescription.add(jsc);
                    jsc.setViewportView(da);

                jpMain.setLayout(new BoxLayout(jpMain, BoxLayout.Y_AXIS));
                jpMain.add(jpUpper);
                jpMain.add(Box.createVerticalStrut(10));
                jpMain.add(jpDescription);

                fs.setComponent(jpMain);
                fs.removeAllButtons();
                fs.addButton("Details", ButtonIDs.BTN_DETAIL, null);
            }
        });
        return stfs;
    }

    /**
     * @return the class that controls and evaluates the {@link JComboBox JComboBoxes}.
     */
    public JCTimeRangeBoxes getTimeRangeBoxes() {
        return jctrb;
    }
}


/**
 * The {@link TableEntryDescriptor} used by {@link FSManagerArticleStatsMain}.
 */
class TEDManagerArticleStatsMain extends AbstractTableEntryDescriptor {

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 2;
    }

    /**
     * @param nIndex the affected column.
     * @return columns' names.
     */
    public String getColumnName(int nIndex) {
        return (new String[]{ "Artikel", "Kategorie"}) [nIndex];
    }

    /**
     * @param nIndex the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass  (int nIndex) {
        return String.class;
    }

    /**
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return columns' values
     */
    public Object getValueAt(Object oRecord, int nIndex) {
        CIArticle article = Conversions.recordToCIArticle(oRecord);
        switch (nIndex) {
            case 0: return article.getArticleName();
            case 1: return article.getCategory();
        }
        return null;
    }

    /**
     * Determines if columns can be sorted by the user.
     *
     * @param nIndex the affected column.
     * @return <ul><li>true: columns can be sorted</li>
     *              <li>false: columns cannot be sorted</li></ul>
     */
    public boolean canSortByColumn(int nIndex) {
        return true;
    }
}

