package market.stdform;

import market.UCustomer;
import market.UMUserBase;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import users.UserManagerFilter;
import users.stdforms.UserTableFormSheet;
import util.swing.AbstractTableEntryDescriptor;

/**
 * This FormSheet displays all customers of the market in a table. Customers can be selected for a
 * detailed view or for deletion.
 */
public class FSManagerCustomerOverview extends UserTableFormSheet {

    /**
     * Creates a {@link UserTableFormSheet}. The look of the table is
     * defined by the {@link TEDManagerCustomerOverview}.
     */
    public FSManagerCustomerOverview() {
        super("Kunden", new UserManagerFilter(UMUserBase.getGlobalBase().getCustomers()),
                null, null, null, new TEDManagerCustomerOverview());
        addContentCreator(new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                fs.removeAllButtons();
                fs.addButton("Daten bearbeiten", ButtonIDs.BTN_EDIT, null);
                fs.addButton("Geschftsbeziehung beenden", ButtonIDs.BTN_DELETE, null);
            }
        });
    }
}

/**
 * The {@link util.swing.TableEntryDescriptor} used by {@link FSManagerCustomerOverview}.
 */
class TEDManagerCustomerOverview extends AbstractTableEntryDescriptor {

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 2;
    }

    /**
     * @param nIndex the affected column.
     * @return columns' names.
     */
    public String getColumnName(int nIndex) {
        return (new String[]{"Name", "Firma"}) [nIndex];
    }

    /**
     * @param nIndex the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass  (int nIndex) {
        return String.class;
    }

    /**
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return columns' values
     */
    public Object getValueAt(Object oRecord, int nIndex) {
        UCustomer usr = (UCustomer)oRecord;
        switch (nIndex) {
            case 0:
                return usr.getSurname() + ", " + usr.getFirstName();
            case 1:
                return usr.getCompany();
        }
        return null;
    }


    /**
     * Determines if columns can be sorted by the user.
     *
     * @param nIndex the affected column.
     * @return <ul><li>true: columns can be sorted</li>
     *              <li>false: columns cannot be sorted</li></ul>
     */
    public boolean canSortByColumn(int nIndex) {
        return true;
    }

}