package market.stdform;

import java.util.Comparator;

import market.Conversions;
import market.SMarket;
import market.UCustomer;
import market.UMUserBase;
import market.event.MarketEventAdapter;
import market.statistics.Statistics;
import market.swing.CmpCustomersNumbers;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import users.UserManagerFilter;
import users.stdforms.UserTableFormSheet;
import util.swing.AbstractTableEntryDescriptor;

/**
 * This FormSheet displays all customers of the market in a table, along with some statistics. From here,
 * a customer whose complete statistics wish to be viewed can be selected.
 */
public class FSManagerCustomerStatsMain extends UserTableFormSheet {

    /**
     * Creates a {@link UserTableFormSheet}. The look of the table is
     * defined by the {@link TEDManagerCustomerStatsMain}.
     */
    public FSManagerCustomerStatsMain() {
        super("Kundenstatistik", new UserManagerFilter(UMUserBase.getGlobalBase().getCustomers()),
                null, null, null, new TEDManagerCustomerStatsMain());
        addContentCreator(new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                fs.removeAllButtons();
                fs.addButton("Details", ButtonIDs.BTN_DETAIL, null);
            }
        });
        //update view when time advances
        SMarket.addEventListener(new MarketEventAdapter() {
            public void timeAdvanced() {
                ((util.swing.TableSorter)getTable().getModel()).fireTableDataChanged();
            }
        });
    }
}

/**
 * The {@link util.swing.TableEntryDescriptor} used by {@link FSManagerCustomerStatsMain}.
 */
class TEDManagerCustomerStatsMain extends AbstractTableEntryDescriptor {

    private Comparator salesVolume = new CmpCustomersNumbers(CmpCustomersNumbers.SALES_VOLUME);
    private Comparator discount = new CmpCustomersNumbers(CmpCustomersNumbers.DISCOUNT);

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 4;
    }

    /**
     * @param nIndex the affected column.
     * @return columns' names.
     */
    public String getColumnName(int nIndex) {
        return (new String[]{"Name", "Mitglied seit", "Anrechenbarer Umsatz", "Rabatt"}) [nIndex];
    }

    /**
     * @param nIndex the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass  (int nIndex) {
        return new Class[] {String.class, String.class, Integer.class, Integer.class}[nIndex];
    }

    /**
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return columns' values
     */
    public Object getValueAt(Object oRecord, int nIndex) {
        UCustomer usr = (UCustomer)oRecord;
        switch (nIndex) {
            case 0:
                return usr.getSurname() + ", " + usr.getFirstName();
            case 1:
                return usr.getDayOfRegistration();
            case 2:
                return Conversions.doubleToCurrency(Statistics.getAllowableCustomerRevenue(usr), " Euro");
            case 3:
                return Conversions.fixedDecimal(100 * usr.getDiscount(), 3) + " %";
        }
        return null;
    }

    /**
     * Determines if columns can be sorted by the user.
     *
     * @param nIndex the affected column.
     * @return <ul><li>true: columns can be sorted</li>
     *              <li>false: columns cannot be sorted</li></ul>
     */
    public boolean canSortByColumn(int nIndex) {
        return true;
    }

    /**
     * @param nIndex the affected column.
     * @return the {@link Comparator} to be used when sorting the column.
     */
    public Comparator getColumnOrder(int nIndex) {
        switch (nIndex) {
            case 2:
                return salesVolume;
            case 3:
                return discount;
        }
        return null;
    }
}
