package market.stdform;

import market.UMUserBase;
import market.UStaffer;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import users.UserManagerFilter;
import users.stdforms.UserTableFormSheet;
import util.swing.AbstractTableEntryDescriptor;

/**
 * This FormSheet displays the whole staff of the market in a table. New workers can be employed and
 * current workers can have their data edited by the manager or even be fired.
 */
public class FSManagerEmployeeOverview extends UserTableFormSheet {

    /**
     * Creates a {@link UserTableFormSheet}. The look of the table is
     * defined by the {@link TEDManagerEmployeeOverview}.
     */
    public FSManagerEmployeeOverview() {
        super("Angestellte", new UserManagerFilter(UMUserBase.getGlobalBase().getStaff()),
                null, null, null, new TEDManagerEmployeeOverview());
        addContentCreator(new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                fs.removeAllButtons();
                fs.addButton("Einstellen", ButtonIDs.BTN_ADD, null);
                fs.addButton("Daten bearbeiten", ButtonIDs.BTN_EDIT, null);
                fs.addButton("Entlassen", ButtonIDs.BTN_DELETE, null);
            }
        });
    }
}

/**
 * The {@link util.swing.TableEntryDescriptor} used by {@link FSManagerEmployeeOverview}.
 */
class TEDManagerEmployeeOverview extends AbstractTableEntryDescriptor {

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 2;
    }

    /**
     * @param nIndex the affected column.
     * @return columns' names.
     */
    public String getColumnName(int nIndex) {
        return (new String[]{"Name", "Abteilung"}) [nIndex];
    }

    /**
     * @param nIndex the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass  (int nIndex) {
        return String.class;
    }

    /**
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return columns' values
     */
    public Object getValueAt(Object oRecord, int nIndex) {
        UStaffer usr = (UStaffer)oRecord;
        switch (nIndex) {
            case 0: return usr.getSurname() + ", " + usr.getFirstName();
            case 1: return usr.getQualification();
        }
        return null;
    }

    /**
     * Determines if columns can be sorted by the user.
     *
     * @param nIndex the affected column.
     * @return <ul><li>true: columns can be sorted</li>
     *              <li>false: columns cannot be sorted</li></ul>
     */
    public boolean canSortByColumn(int nIndex) {
        return true;
    }
}