package market.stdform;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Comparator;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellEditor;

import market.CFilter;
import market.CIArticle;
import market.CSOffer;
import market.Conversions;
import market.SMarket;
import market.swing.CmpNumbers;
import market.swing.JTADescriptionArea;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import util.swing.AbstractTableEntryDescriptor;
import util.swing.TableEntryDescriptor;
import data.IntegerValue;
import data.stdforms.SingleTableFormSheet;
import data.swing.CountingStockTableModel;

/**
 * This FormSheet displays the market's stock. The contained table allows editing of the bid
 * when the market is closed.
 */
public class FSManagerOffer {

    /**
     * Creates a {@link SingleTableFormSheet} with some extra components. The look of the table is
     * defined by the {@link TEDManagerOffer}.
     *
     * @return the created SingleTableFormSheet.
     */
    public static SingleTableFormSheet create() {
        final TableEntryDescriptor ted = new TEDManagerOffer();
        final CSOffer source = SMarket.getOffer();
        final SingleTableFormSheet stfs = SingleTableFormSheet.create(
                "Bestands- und Preisbersicht",
                source,
                null,                         //UIGate
                true,                         //show zeros
                ted                           //TED
        );
        stfs.addContentCreator(new FormSheetContentCreator() {
            protected void createFormSheetContent(final FormSheet fs) {
                JPanel jpMain = new JPanel();
                JPanel jpTable = new JPanel();
                JPanel jpFilter = new JPanel();
                JPanel jpTableAndFilter = new JPanel();
                JPanel jpDescription = new JPanel();
                JScrollPane jsc = new JScrollPane();
                GridBagConstraints c = new GridBagConstraints();
                GridBagLayout gridbag = new GridBagLayout();
                final JComboBox jcbFilter = new JComboBox(SMarket.getArticleCategories());
                jcbFilter.addItemListener(new ItemListener() {
                    public void itemStateChanged(ItemEvent e) {
                        if (e.getStateChange() == e.SELECTED) {
                            int i = jcbFilter.getSelectedIndex();
                            CFilter cf = new CFilter(source.getCatalog(null), i); //filter CatalogItems
                            CSOffer cs = (CSOffer)source.clone();
                            try {
                                cs.changeArticleCatalog(cf);  //make new CountingStock from
                            }                                 //filtered CatalogItems
                            catch (Exception ex){
                                ex.printStackTrace();
                            }
                            //send new created CountingStock to table
                            stfs.setTableModel(new CountingStockTableModel(cs, null, null, true, ted));
                        }
                    }
                });
                final JTADescriptionArea da = new JTADescriptionArea();
                //add Listener to table that reacts on selection change
                stfs.getTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                    public void valueChanged(ListSelectionEvent e) {
                        if (!e.getValueIsAdjusting()) {
                            da.setDescription(Conversions.recordToCIArticle(stfs.getSelectedRecord()));
                        }
                    }
                });
                jpTableAndFilter.setLayout(gridbag);
                    c.gridx = 0;
                    c.weightx = 1;
                    c.weighty = 1;
                    c.gridy = 0;
                    c.fill = GridBagConstraints.BOTH;
                  gridbag.setConstraints(jpTable, c);
                    c.gridx = 1;
                    c.weightx = 0;
                    c.weighty = 1;
                    c.gridy = 0;
                    c.fill = GridBagConstraints.BOTH;
                  gridbag.setConstraints(jpFilter, c);
                jpTableAndFilter.add(jpTable);
                jpTable.setLayout(new BoxLayout(jpTable, BoxLayout.X_AXIS));
                jpTable.add(fs.getComponent());
                jpTableAndFilter.add(jpFilter);
                    jpFilter.add(jcbFilter);
                jpDescription.setLayout(new BoxLayout(jpDescription, BoxLayout.X_AXIS));
                jpDescription.add(jsc);
                    jsc.setViewportView(da);

                jpMain.setLayout(new BoxLayout(jpMain, BoxLayout.Y_AXIS));
                jpMain.add(jpTableAndFilter);
                jpMain.add(Box.createVerticalStrut(10));
                jpMain.add(jpDescription);

                fs.setComponent(jpMain);
                fs.removeAllButtons();
            }
        });
        return stfs;
    }
}

/**
 * The {@link util.swing.TableEntryDescriptor} used by {@link FSManagerOffer}.
 */
class TEDManagerOffer extends AbstractTableEntryDescriptor {

    private Comparator sortOffer = new CmpNumbers(CmpNumbers.OFFER);
    private Comparator sortBid   = new CmpNumbers(CmpNumbers.BID);
    private Comparator sortCount = new CmpNumbers(CmpNumbers.COUNT);

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 4;
    }

    /**
     * @param nIndex the affected column.
     * @return columns' names.
     */
    public String getColumnName(int nIndex) {
        return (new String[]{ "Artikel", "Einkaufspreis", "Verkaufspreis", "Anzahl"}) [nIndex];
    }

    /**
     * @param nIndex the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass  (int nIndex) {
        return (new Class[] {String.class, Number.class, Number.class, Number.class}) [nIndex];
    }

    /**
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return columns' values
     */
    public Object getValueAt(Object oRecord, int nIndex) {
        CountingStockTableModel.Record stockitem = null;
        CIArticle article = Conversions.recordToCIArticle(oRecord);
        int count = ((CountingStockTableModel.Record)oRecord).getCount();
        switch (nIndex) {
            case 0: return article.getArticleName();
            case 1: return Conversions.doubleToCurrency(article.getOffer());
            case 2: return Conversions.doubleToCurrency(article.getBid());
            case 3: return new Integer(count).toString();
        }
        return null;
    }

    /**
     * Determines if columns can be sorted by the user.
     *
     * @param nIndex the affected column.
     * @return <ul><li>true: columns can be sorted</li>
     *              <li>false: columns cannot be sorted</li></ul>
     */
    public boolean canSortByColumn(int nIndex) {
        return true;
    }

    /**
     * @param nIndex the affected column.
     * @return the {@link Comparator} to be used when sorting the column.
     */
    public Comparator getColumnOrder(int nIndex) {
        switch(nIndex) {
            case 1: return sortOffer;
            case 2: return sortBid;
            case 3: return sortCount;
        }
        return null;
    }

    /**
     * Determines editable columns.
     *
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return <ul><li>true: column's cells are editable</li>
     *              <li>false: column's cells are not editable</li></ul>
     */
    public boolean isElementEditable(Object oRecord, int nIndex) {
        //3rd column, but only if market is not open
        return nIndex == 2 && !SMarket.isOpen();
    }

    /**
     * Assigns the value to a table cell.
     *
     * @param oRecord the affected table record
     * @param nIndex the affected column.
     * @param oValue the new Object that is tried to be set as the table cell's content.
     */
    public void setValueAt(Object oRecord, int nIndex, Object oValue) {
        String newValueString = Conversions.convertComma((String)oValue);

        if (newValueString != null) {
            int newPrice = new Double(Double.parseDouble(newValueString)*100).intValue();
            if (newPrice >= 0) {
                Conversions.recordToCIArticle(oRecord).setBid(new IntegerValue(newPrice));
            }
            SMarket.getMonthlySalesStats().get(Conversions.recordToCIArticle(oRecord).getName()).newPriceSet(
                    SMarket.getTime(), newPrice);
        }
    }

    /**
     * Sets a {@link JTextField} as the editor for the editable column.
     *
     * @param nIndex the affected column.
     * @return TableCellEditor of the affected column.
     */
    public TableCellEditor getCellEditor(int nIndex) {
        if (nIndex == 2) {
             return new DefaultCellEditor(new JTextField());
        }
        return null;
    }
}