package market.stdform;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Comparator;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import market.CIArticle;
import market.CIOpenPurchaseOrders;
import market.Conversions;
import market.swing.CmpNumbers;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import util.swing.AbstractTableEntryDescriptor;
import data.stdforms.SingleTableFormSheet;
import data.swing.CountingStockTableModel;

/**
 * This FormSheet displays details of a market's order that has not yet arrived.
 */
public class FSManagerOpenPurchaseOrderDetail {

    /**
     * Creates a {@link SingleTableFormSheet} with some extra components. The look of the table is
     * defined by the {@link TEDManagerOpenPurchaseOrderDetail}.
     *
     * @return the created SingleTableFormSheet.
     */
    public static SingleTableFormSheet create(final CIOpenPurchaseOrders opo) {
        final SingleTableFormSheet stfs = SingleTableFormSheet.create(
                "Bestellung " + opo.getOrderNumber() + " vom " + opo.getDate(),
                opo.getOrders(),
                null,                                     //DataBasket
                new TEDManagerOpenPurchaseOrderDetail()   //TED
        );
        stfs.addContentCreator(new FormSheetContentCreator() {
            protected void createFormSheetContent(final FormSheet fs) {
                //define components
                JPanel jpMain = new JPanel();
                JPanel jpTable = new JPanel();
                JPanel jpInfo = new JPanel();
                JScrollPane jsp = (JScrollPane)fs.getComponent();
                GridBagConstraints c = new GridBagConstraints();
                GridBagLayout gridbag = new GridBagLayout();
                //add components
                jpTable.setLayout(gridbag);
                      c.gridx = 0;
                      c.weightx = 1;
                      c.weighty = 1;
                      c.gridy = 0;
                      c.fill = GridBagConstraints.BOTH;
                    gridbag.setConstraints(jsp, c);
                jpTable.add(jsp);
                jpInfo.setLayout(new BoxLayout(jpInfo, BoxLayout.Y_AXIS));
                jpInfo.add(new JLabel("Gesamtpreis:   " +
                        Conversions.doubleToCurrency(opo.getOrdersValue().doubleValue(), " Euro")));
                jpInfo.add(new JLabel("Lieferung kommt in " + opo.getDaysTillArrival() + " Tagen an."));

                jpMain.setLayout(new BoxLayout(jpMain, BoxLayout.Y_AXIS));
                jpMain.add(jpTable);
                jpMain.add(Box.createVerticalStrut(10));
                jpMain.add(jpInfo);
                fs.setComponent(jpMain);
                fs.removeAllButtons();
                fs.addButton("Zurck", ButtonIDs.BTN_BACK, null);
            }
        });
        return stfs;
    }
}

/**
 * The {@link util.swing.TableEntryDescriptor} used by {@link FSManagerOpenPurchaseOrderDetail}.
 */
class TEDManagerOpenPurchaseOrderDetail extends AbstractTableEntryDescriptor {

    private Comparator sortOfferNoSum = new CmpNumbers(CmpNumbers.OFFER, CmpNumbers.SINGLE);
    private Comparator sortOfferSum = new CmpNumbers(CmpNumbers.OFFER, CmpNumbers.SUMMED);
    private Comparator sortCount = new CmpNumbers(CmpNumbers.COUNT);

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 5;
    }

    /**
     * @param nIndex the affected column.
     * @return columns' names.
     */
    public String getColumnName(int nIndex) {
        return (new String[]{ "Artikel", "Kategorie", "Einzeleinkaufspreis", "Anzahl", "Summe"}) [nIndex];
    }

    /**
     * @param nIndex the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass  (int nIndex) {
        return (new Class[] {String.class, String.class, Number.class, Number.class, Number.class}) [nIndex];
    }

    /**
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return columns' values
     */
    public Object getValueAt(Object oRecord, int nIndex) {
        CIArticle article = Conversions.recordToCIArticle(oRecord);
        int count = ((CountingStockTableModel.Record)oRecord).getCount();
        switch (nIndex) {
            case 0: return article.getArticleName();
            case 1: return article.getCategory();
            case 2: return Conversions.doubleToCurrency(article.getOffer());
            case 3: return new Integer(count).toString();
            case 4: return Conversions.doubleToCurrency(count * article.getOffer());
        }
        return null;
    }

    /**
     * Determines if columns can be sorted by the user.
     *
     * @param nIndex the affected column.
     * @return <ul><li>true: columns can be sorted</li>
     *              <li>false: columns cannot be sorted</li></ul>
     */
    public boolean canSortByColumn(int nIndex) {
        return true;
    }

    /**
     * @param nIndex the affected column.
     * @return the {@link Comparator} to be used when sorting the column.
     */
    public Comparator getColumnOrder(int nIndex) {
        switch(nIndex) {
            case 2: return sortOfferNoSum;
            case 3: return sortCount;
            case 4: return sortOfferSum;
        }
        return null;
    }
}