package market.stdform;

import java.util.Comparator;

import market.CIOpenPurchaseOrders;
import market.Conversions;
import market.SMarket;
import market.swing.CmpOpoNumbers;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import util.swing.AbstractTableEntryDescriptor;
import data.stdforms.SingleTableFormSheet;

/**
 * This FormSheet displays an overview of the market's purchases that have not yet arrived.
 */
public class FSManagerOpenPurchaseOrderMain {

    /**
     * Creates a {@link SingleTableFormSheet}. The look of the table is defined by the
     * {@link TEDManagerOpenPurchaseOrderMain}.
     *
     * @return the created SingleTableFormSheet.
     */
    public static SingleTableFormSheet create() {

        final SingleTableFormSheet stfs = SingleTableFormSheet.create(
            "Erwartete Lieferungen",
            SMarket.getOpenPurchaseOrder(),
            null,                                     //DataBasket
            new TEDManagerOpenPurchaseOrderMain()     //TED
        );
        stfs.addContentCreator(new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                fs.removeAllButtons();
                fs.addButton("Details", ButtonIDs.BTN_DETAIL, null);
            }
        });
        return stfs;
    }
}

/**
 * The {@link util.swing.TableEntryDescriptor} used by {@link FSManagerOpenPurchaseOrderMain}.
 */
class TEDManagerOpenPurchaseOrderMain extends AbstractTableEntryDescriptor {

    private Comparator sortValue = new CmpOpoNumbers();

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 3;
    }

    /**
     * @param nIndex the affected column.
     * @return columns' names.
     */
    public String getColumnName(int nIndex) {
        return (new String[]{ "Bestelldatum", "Gesamtsumme", "Tage bis Ankunft"}) [nIndex];
    }

    /**
     * @param nIndex the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass  (int nIndex) {
        return Number.class;
    }

    /**
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return columns' values
     */
    public Object getValueAt(Object oRecord, int nIndex) {
        CIOpenPurchaseOrders opo = (CIOpenPurchaseOrders)oRecord;
        switch (nIndex) {
            case 0: return opo.getName();
            case 1: return Conversions.doubleToCurrency(opo.getOrdersValue().doubleValue(), " Euro");
            case 2: return opo.getValue();
        }
        return null;
    }

    /**
     * Determines if columns can be sorted by the user.
     *
     * @param nIndex the affected column.
     * @return <ul><li>true: columns can be sorted</li>
     *              <li>false: columns cannot be sorted</li></ul>
     */
    public boolean canSortByColumn(int nIndex) {
        return true;
    }

    /**
     * @param nIndex the affected column.
     * @return the {@link Comparator} to be used when sorting the column.
     */
    public Comparator getColumnOrder(int nIndex) {
        switch(nIndex) {
            case 0: return null;
            case 1: return sortValue;
            case 2: return null;
        }
        return null;
    }
}