package market.stdform;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import market.Conversions;
import market.SMarket;
import market.VCPositiveDouble;
import market.VCPositiveInteger;
import market.swing.ComponentFactory;
import market.swing.JTFCheckable;
import sale.FormSheet;
import sale.FormSheetContentCreator;

/**
 * This FormSheet is used by the manager to set special calculation variables such as the maximal discount
 * for customers or the monthly costs of the market.
 */
public class FSManagerOptions extends FormSheet {

    public static final int JTFC_DISCOUNT_RANGE = 0;
    public static final int JTFC_DISCOUNT_VALUE = 1;
    public static final int JTFC_MAX_DISCOUNT = 2;
    public static final int JTFC_FRACTION_OF_WAGES = 3;
    public static final int JTFC_SENIORITY = 4;
    public static final int JTFC_COSTS = 5;

    private JTFCheckable jtfcDiscountRange = new JTFCheckable(
            JTFC_DISCOUNT_RANGE, new VCPositiveInteger("Anrechnung"), 10);
    private JTFCheckable jtfcDiscountValue = new JTFCheckable(
            JTFC_DISCOUNT_VALUE, new VCPositiveInteger("Umsatz"), 10);
    private JTFCheckable jtfcMaxDiscount = new JTFCheckable(
            JTFC_MAX_DISCOUNT, new VCPositiveDouble("Maximaler Rabatt"), 10);
    private JTFCheckable jtfcFractionOfWages = new JTFCheckable(
            JTFC_FRACTION_OF_WAGES, new VCPositiveDouble("Lohnanteil"), 10);
    private JTFCheckable jtfcSeniority = new JTFCheckable(
            JTFC_SENIORITY, new VCPositiveDouble("Betriebszugehrigkeit"), 10);
    private JTFCheckable jtfcCosts = new JTFCheckable(
            JTFC_COSTS, new VCPositiveDouble("Sonstige Kosten"), 10);

    public FSManagerOptions() {
        super("Einstellungen", null);
        addContentCreator(new FormSheetContentCreator() {protected void createFormSheetContent(FormSheet fs) {
            //define components
            JPanel jpMain = new JPanel();
            JPanel jpData = new JPanel();
            JPanel jpDiscount = new JPanel();
            JPanel jpEmployees = new JPanel();
            JPanel jpCostCentres = new JPanel();
            GridBagLayout gridBag = new GridBagLayout();
            GridBagConstraints c = new GridBagConstraints();
            jpMain.setLayout(gridBag);
                c.weighty = 1;
                c.anchor = GridBagConstraints.CENTER;
            gridBag.setConstraints(jpData, c);

            jpMain.add(jpData);
                jpData.setLayout(new BoxLayout(jpData, BoxLayout.Y_AXIS));
                jpData.add(jpDiscount);
                    jpDiscount.setLayout(new BoxLayout(jpDiscount, BoxLayout.Y_AXIS));
                    jpDiscount.setBorder(ComponentFactory.createInsetBorder("Kundenrabatt"));
                    jpDiscount.setLayout(new GridLayout(3, 3));
                    jpDiscount.add(new JLabel("Anrechnung:  "));
                    jpDiscount.add(jtfcDiscountRange);
                    jpDiscount.add(new JLabel("   Monate"));
                    jpDiscount.add(new JLabel("Umsatz fr 1% Rabatt:  "));
                    jpDiscount.add(jtfcDiscountValue);
                    jpDiscount.add(new JLabel("   Euro"));
                    jpDiscount.add(new JLabel("Maximaler Rabatt:  "));
                    jpDiscount.add(jtfcMaxDiscount);
                    jpDiscount.add(new JLabel("   Prozent"));
                jpData.add(Box.createVerticalStrut(10));
                jpData.add(jpEmployees);
                    jpEmployees.setLayout(new GridLayout(2, 3));
                    jpEmployees.setBorder(ComponentFactory.createInsetBorder(
                            "Faktoren zur Berechnung des Entlassungsausgleichs"));
                    jpEmployees.add(new JLabel("Lohnanteil:  "));
                    jpEmployees.add(jtfcFractionOfWages);
                    jpEmployees.add(new JLabel("   Prozent"));
                    jpEmployees.add(new JLabel("Betriebszugehrigkeit:  "));
                    jpEmployees.add(jtfcSeniority);
                    jpEmployees.add(new JLabel("   Prozent"));
                jpData.add(Box.createVerticalStrut(10));
                jpData.add(jpCostCentres);
                    jpCostCentres.setBorder(ComponentFactory.createInsetBorder("Kostenstellen"));
                    jpCostCentres.setLayout(new GridLayout(1, 3));
                    jpCostCentres.add(new JLabel("Sonstige Kosten: "));
                    jpCostCentres.add(jtfcCosts);
                    jpCostCentres.add(new JLabel("   Euro"));

            fs.setComponent(jpMain);
            fs.removeAllButtons();
            fs.addButton("bernehmen", ButtonIDs.BTN_ACCEPT, null);

            jtfcDiscountRange.setText(Integer.toString(SMarket.getOptions().getDiscountRange()));
            jtfcDiscountValue.setText(Integer.toString(SMarket.getOptions().getDiscountValue()));
            jtfcMaxDiscount.setText(Conversions.fixedDecimal(100 * SMarket.getOptions().getMaxDiscount(), 3));
            jtfcFractionOfWages.setText(Conversions.fixedDecimal(100 * SMarket.getOptions().
                    getFractionOfWages(), 3));
            jtfcSeniority.setText(Conversions.fixedDecimal(100 * SMarket.getOptions().getTimeOfEmployment(), 3));
            jtfcCosts.setText(Conversions.doubleToCurrency(SMarket.getMonthlySalesStats().getCosts()));
        }});
    }

}