package market.stdform;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import market.Conversions;
import market.SMarket;
import market.statistics.CSalesStats;
import market.statistics.Statistics;
import market.swing.ComponentFactory;
import market.swing.JCTimeRangeBoxes;
import sale.FormSheet;
import sale.FormSheetContentCreator;

/**
 * This FormSheet displays receipts and expenditures of the market. The range of time for which they
 * should be computed is also adjustable here.
 */
public class FSManagerOverallStats extends FormSheet {

    private JTextField jtfCurrentCapital;
    private JTextField jtfSales;
    private JTextField jtfOrders;
    private JTextField jtfWages;
    private JTextField jtfMisc;
    private JTextField jtfSum;
    private JTextField jtfSumRevenue;
    private JTextField jtfSumCosts;

    private JComboBox jcbFromYear, jcbToYear, jcbFromMonth, jcbToMonth;
    private JCTimeRangeBoxes jctrb;

    public FSManagerOverallStats() {
        super("Umsatzstatistik", null);
        addContentCreator(new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                JPanel jpMain = new JPanel();
                JPanel jpCurrentCapital = new JPanel();
                JPanel jpTimeSelect = new JPanel();
                JPanel jpData = new JPanel();
                JPanel jpDataRevenue = new JPanel();
                JPanel jpDataCosts = new JPanel();
                JPanel jpDataRevenueDetail = new JPanel();
                JPanel jpDataCostsDetail = new JPanel();
                JPanel jpSumRevenue = new JPanel();
                JPanel jpSumCosts = new JPanel();
                JPanel jpFrom = new JPanel();
                JPanel jpTo = new JPanel();
                JPanel jpSummarize = new JPanel();
                JButton jbShow = new JButton("Anzeigen");
                jtfCurrentCapital = ComponentFactory.createTextField("", 10, false,
                        ComponentFactory.RIGHT, false);
                jtfSales = ComponentFactory.createTextField("", 10, false,
                        ComponentFactory.RIGHT, false);
                jtfOrders = ComponentFactory.createTextField("", 10, false,
                        ComponentFactory.RIGHT, Color.red, false);
                jtfWages = ComponentFactory.createTextField("", 10, false,
                        ComponentFactory.RIGHT, Color.red, false);
                jtfMisc = ComponentFactory.createTextField("", 10, false,
                        ComponentFactory.RIGHT, Color.red, false);
                jtfSum = ComponentFactory.createTextField("", 10, false,
                        ComponentFactory.RIGHT, false);
                jtfSumRevenue = ComponentFactory.createTextField("", 10, false,
                        ComponentFactory.RIGHT, false);
                jtfSumCosts = ComponentFactory.createTextField("", 10, false,
                        ComponentFactory.RIGHT, Color.red, false);
                jctrb = new JCTimeRangeBoxes();
                jcbFromYear = jctrb.getFromYearBox();
                jcbToYear = jctrb.getToYearBox();
                jcbFromMonth = jctrb.getFromMonthBox();
                jcbToMonth = jctrb.getToMonthBox();

                GridBagLayout gridBag = new GridBagLayout();
                jpMain.setLayout(gridBag);
                GridBagConstraints c = new GridBagConstraints();
                  c.gridy = 0;
                  c.weighty = 0.2;
                  c.insets = new Insets(10,0,0,0);
                  c.anchor = GridBagConstraints.CENTER;
                gridBag.setConstraints(jpCurrentCapital, c);
                  c.gridy = 1;
                  c.weighty = 0.1;
                  c.insets = new Insets(10,0,0,0);
                  c.anchor = GridBagConstraints.CENTER;
                gridBag.setConstraints(jpTimeSelect, c);
                  c.gridy = 2;
                  c.weighty = 0.7;
                  c.anchor = GridBagConstraints.CENTER;
                gridBag.setConstraints(jpData, c);
                  c.gridy = 3;
                  c.weighty = 0;
                  c.ipadx = 0;
                  c.ipady = 0;
                  c.insets = new Insets(10,0,0,0);
                  c.anchor = GridBagConstraints.SOUTH;
                gridBag.setConstraints(jpSummarize, c);


                jpCurrentCapital.setBorder(ComponentFactory.createInsetBorder("Derzeitiges Vermgen"));
                jpCurrentCapital.add(jtfCurrentCapital);
                    jtfCurrentCapital.setFont(new Font("",0,20));

                jpTimeSelect.setBorder(ComponentFactory.createInsetBorder(
                        ComponentFactory.createInsetBorder(
                        "Auswahl des Zeitraums fr die Umsatzstatistik"), 15, 15, 0, 15));
                jpTimeSelect.setLayout(new BoxLayout(jpTimeSelect, 0));
                jpTimeSelect.add(jpFrom);
                    jpFrom.setBorder(ComponentFactory.createInsetBorder("von"));
                    jpFrom.add(jcbFromMonth);
                    jpFrom.add(jcbFromYear);
                jpTimeSelect.add(Box.createHorizontalStrut(20));
                jpTimeSelect.add(jpTo);
                    jpTo.setBorder(ComponentFactory.createInsetBorder("bis"));
                    jpTo.add(jcbToMonth);
                    jpTo.add(jcbToYear);
                jpTimeSelect.add(Box.createHorizontalStrut(20));
                jpTimeSelect.add(jbShow);
                    jbShow.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            fillTextFields();
                        }
                    });

                jpData.setLayout(new BoxLayout(jpData, 0));
                jpData.add(jpDataRevenue);
                    jpDataRevenue.setLayout(new BoxLayout(jpDataRevenue, 1));
                    jpDataRevenue.add(jpDataRevenueDetail);
                        jpDataRevenueDetail.setBorder(ComponentFactory.createInsetBorder("Einnahmen"));
                        jpDataRevenueDetail.setLayout(new GridLayout(4,2));
                        jpDataRevenueDetail.add(new JLabel("Verkauf"));
                        jpDataRevenueDetail.add(jtfSales);
                        jpDataRevenueDetail.add(new JLabel(""));
                        jpDataRevenueDetail.add(new JLabel(""));
                        jpDataRevenueDetail.add(new JLabel(""));
                        jpDataRevenueDetail.add(new JLabel(""));
                        jpDataRevenueDetail.add(new JLabel(""));
                        jpDataRevenueDetail.add(new JLabel(""));
                    jpDataRevenue.add(jpSumRevenue);
                        jpSumRevenue.setBorder(BorderFactory.createEmptyBorder(15,15,15,15));
                        jpSumRevenue.setLayout(new GridLayout(1,2));
                        jpSumRevenue.add(new JLabel("Summe"));
                        jpSumRevenue.add(jtfSumRevenue);
                jpData.add(jpDataCosts);
                    jpDataCosts.setLayout(new BoxLayout(jpDataCosts, 1));
                    jpDataCosts.add(jpDataCostsDetail);
                        jpDataCostsDetail.setBorder(ComponentFactory.createInsetBorder("Ausgaben"));
                        jpDataCostsDetail.setLayout(new GridLayout(4,2));
                        jpDataCostsDetail.add(new JLabel("Einkufe"));
                        jpDataCostsDetail.add(jtfOrders);
                        jpDataCostsDetail.add(new JLabel("Lohnkosten"));
                        jpDataCostsDetail.add(jtfWages);
                        jpDataCostsDetail.add(new JLabel("Sonstiges"));
                        jpDataCostsDetail.add(jtfMisc);
                        jpDataCostsDetail.add(new JLabel(""));
                        jpDataCostsDetail.add(new JLabel(""));
                    jpDataCosts.add(jpSumCosts);
                        jpSumCosts.setBorder(BorderFactory.createEmptyBorder(15,15,15,15));
                        jpSumCosts.setLayout(new GridLayout(1,2));
                        jpSumCosts.add(new JLabel("Summe"));
                        jpSumCosts.add(jtfSumCosts);

                jpSummarize.add(new JLabel("Gesamtergebnis: "));
                jpSummarize.add(jtfSum);
                    jtfSum.setFont(new Font("",0,20));

                jpMain.add(jpCurrentCapital);
                jpMain.add(jpTimeSelect);
                jpMain.add(jpData);
                jpMain.add(jpSummarize);

                fs.setComponent(jpMain);
                fs.removeAllButtons();
                fillTextFields();
            }
        });
    }

    /**
     * Evaluates the {@link JComboBox JComboBoxes}, computes the according statistics and updates
     * the text fields.
     */
    private void fillTextFields() {
        if (jctrb.isValidTimeRange()) {
            CSalesStats currentStats = Statistics.getOverallStats(jctrb.getFromMonth(), jctrb.getFromYear(),
                    jctrb.getToMonth(), jctrb.getToYear());
            jtfCurrentCapital.setText(Conversions.valueToCurrency(SMarket.getAccount(), " Euro"));
            jtfSum.setForeground(Conversions.valueToInt(SMarket.getAccount()) < 0 ? Color.red : Color.black);
            int sales = currentStats.getRevenue();
            int sumRevenue = sales;
            jtfSales.setText(Conversions.doubleToCurrency(sales, " Euro"));
            jtfSumRevenue.setText(Conversions.doubleToCurrency(sumRevenue, " Euro"));
            int orderCosts = currentStats.getOrderCosts();
            int wages = currentStats.getWages();
            int misc = currentStats.getCosts();
            int sumCosts = orderCosts + wages + misc;
            jtfOrders.setText(Conversions.doubleToCurrency(orderCosts, " Euro"));
            jtfWages.setText(Conversions.doubleToCurrency(wages, " Euro"));
            jtfMisc.setText(Conversions.doubleToCurrency(misc, " Euro"));
            jtfSumCosts.setText(Conversions.doubleToCurrency(sumCosts, " Euro"));
            int profit = sumRevenue - sumCosts;
            jtfSum.setText(Conversions.doubleToCurrency(profit, " Euro"));
            jtfSum.setForeground(profit < 0 ? Color.red : Color.black);
        } else {
            JOptionPane.showMessageDialog(getComponent(), "Ungltiger Zeitraum.");
        }
    }
}
