package market.stdform;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import market.Conversions;
import market.VCDummy;
import market.VCPositiveDouble;
import market.swing.ComponentFactory;
import market.swing.JTFCheckable;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import data.Value;

/**
 * This FormSheet displays the bill of a customer`s buy
 */
public class FSSellerBill extends FormSheet{

    public static final int JTFC_DISCOUNT = 0;
    public static final int JTFC_ENDSUM = 1;

    private Value v_sum;
    private double discount;
    private Value v_endSum;
    private JTFCheckable jtfc_discount;
    private JTFCheckable jtfc_endSum;

    /**
     * @param v the value of customer's current purchase.
     * @param d the customer's current discount, it covers values from 0 to 1.
     */
    public FSSellerBill(Value v, double d) {
        super("Rechnung", null);
        v_sum = v;
        discount = d;
        setEndSum(discount);
        this.addContentCreator(getFSCC());
        jtfc_discount.addKeyListener(new KeyAdapter(){
            public void keyPressed(KeyEvent e) {
                if(e.getKeyCode()==KeyEvent.VK_ENTER){
                    setNewDiscount();
                }
            }
        });
    }

    /**
     * @return a new {@link FSCheckable} that uses a FSSellerBill as argument.
     *
     * @param v the value of customer's current purchase.
     * @param d the customer's current discount, it covers values from 0 to 1.
     */
    public static FSCheckable create(Value v, double d){
        FormSheet fs = new FSSellerBill(v, d);
        return new FSCheckable(fs);
    }

    /**
     * @return the FormSheetContentCreator of this FormSheet class.
     */
    private FormSheetContentCreator getFSCC(){
        return new FormSheetContentCreator(){
            protected void createFormSheetContent(FormSheet fs) {

                //JPanel Initialization
                JPanel jp_content = new JPanel();
                JPanel jp_main = new JPanel();
                JPanel jp_amounts = new JPanel();
                JPanel jp_paymentMode = new JPanel();

                GridBagLayout gridbag = new GridBagLayout();
                GridBagConstraints c = new GridBagConstraints();
                jp_content.setLayout(gridbag);
                    c.gridy = 1;
                    c.weighty = 1;
                gridbag.setConstraints(jp_main, c);

                jp_main.setLayout(new BoxLayout(jp_main, BoxLayout.Y_AXIS));
                jp_amounts.setLayout(new GridLayout(3,3));
                jp_amounts.setAlignmentX(Box.CENTER_ALIGNMENT);
                jp_paymentMode.setLayout(new GridLayout(2,1));
                jp_paymentMode.setBorder(ComponentFactory.createInsetBorder("Zahlungsart"));
                jp_paymentMode.setAlignmentX(Box.CENTER_ALIGNMENT);

                //TextField Initialisation
                JTextField jtf_sum = new JTextField(Conversions.valueToCurrency(v_sum));
                jtfc_discount = new JTFCheckable(JTFC_DISCOUNT, new VCPositiveDouble("Rabatt", false), 3);
                jtfc_discount.setText(Conversions.fixedDecimal(100 * discount, 3));
                jtfc_endSum = new JTFCheckable(JTFC_ENDSUM, new VCDummy("Endsumme"), 7);
                jtfc_endSum.setText(Conversions.valueToCurrency(v_endSum));
                jtf_sum.setEditable(false);
                jtfc_endSum.setEditable(false);

                //ButtonGroup to select mode of payment
                JRadioButton jrb_cash = new JRadioButton("Bar", true);
                JRadioButton jrb_card = new JRadioButton("Geldkarte", false);
                ButtonGroup group = new ButtonGroup();
                group.add(jrb_cash);
                group.add(jrb_card);

                //Adds Components to JPanels
                jp_amounts.add(new JLabel("Summe: "));
                jp_amounts.add(jtf_sum);
                jp_amounts.add(new JLabel(" Euro"));
                jp_amounts.add(new JLabel("Rabatt: "));
                jp_amounts.add(jtfc_discount);
                jp_amounts.add(new JLabel(" %"));
                jp_amounts.add(new JLabel("Rechnungsbetrag: "));
                jp_amounts.add(jtfc_endSum);
                jp_amounts.add(new JLabel(" Euro"));

                jp_paymentMode.add(jrb_cash);
                jp_paymentMode.add(jrb_card);

                jp_main.add(jp_amounts);
                jp_main.add(Box.createVerticalStrut(50));
                jp_main.add(jp_paymentMode);

                jp_content.add(jp_main);

                //Adds components and Buttons to the FormSheet
                fs.setComponent(jp_content);
                fs.removeAllButtons();
                fs.addButton("Zahlung", ButtonIDs.BTN_ACCEPT, null);
                fs.addButton("Zurck", ButtonIDs.BTN_BACK, null);
            }
        };
    }

    /**
     * Updates the TextFields containing discount and end-sum, using entry in field discount.
     */
    private void setNewDiscount(){
        if(jtfc_discount.hasValidValue()){
            double oldDisc = discount;
            discount = Conversions.round(
                    Double.valueOf(Conversions.convertComma(
                    jtfc_discount.getText())).doubleValue()/100, 5);
            if (discount > 1) {   //do not give more than 100 percent discount
                discount = oldDisc;
            }
            setEndSum(discount);
            jtfc_discount.setText(Conversions.fixedDecimal(100 * discount, 3));
            jtfc_endSum.setText(Conversions.valueToCurrency(v_endSum));
        } else {
            jtfc_discount.setText(Conversions.fixedDecimal(100 * discount, 3));
        }
    }

    /**
     * Sets new end-sum using sum and given discount.
     */
    private void setEndSum(double discount){
        if(discount==0) v_endSum = v_sum;
        else v_endSum = v_sum.subtract(v_sum.multiply(discount));
    }

    /**
     * @return the final value the customer has to pay.
     */
    public Value getEndSum(){
        return v_endSum;
    }
}
