package market.stdform;

import market.SICustomer;
import market.UCustomer;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import users.UserManager;
import util.swing.AbstractTableEntryDescriptor;
import util.swing.TableEntryDescriptor;
import data.StoringStock;
import data.stdforms.SingleTableFormSheet;


/**
 * This FormSheet displays the customers who are waiting at the till in a table,
 * with the customer who waits the longest time at the top.
 */
public class FSSellerCustomerTable {

    /**
     * @return the customer table.
     *
     * @param ss the StoringStock  containing the waiting customers  in form of {@link SICustomer}.
     */
    public static SingleTableFormSheet getCustomerTable(StoringStock ss){

        SingleTableFormSheet stfs = SingleTableFormSheet.create(
                "Kunden-Warteschlange",
                ss,
                null,
                getTED());

        stfs.addContentCreator(new FormSheetContentCreator() {
            protected void createFormSheetContent(final FormSheet fs) {
                fs.removeAllButtons();
                fs.addButton("Kunden bedienen", ButtonIDs.BTN_OK, null);
                fs.addButton("Auftrag stornieren", ButtonIDs.BTN_CANCEL, null);
            }}
        );
        return stfs;
    }

    /**
     * @return the TableEntryDescriptor of the table, declaring two columns: the userID and full name of the customer.
     */
    private static TableEntryDescriptor getTED(){
        AbstractTableEntryDescriptor ted = new AbstractTableEntryDescriptor(){
            public int getColumnCount() {
                return 2;
            }

            public String getColumnName(int nIdx) {
                return (new String[]{ "KundenID", "Name"}) [nIdx];
            }

            public Class getColumnClass(int nIdx) {
                return (new Class[] {String.class, String.class}) [nIdx];
            }

            public Object getValueAt(Object oRecord, int nIdx) {
                UCustomer customer = (UCustomer)UserManager.getGlobalUM().getUser(((SICustomer)oRecord).getName());
                switch(nIdx){
                    case 0: return customer.getName();
                    case 1: return customer.getFullName();
                }
                return null;
            }
        };
        return ted;
    }
}
