package market.stdform;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Comparator;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import market.CIArticle;
import market.Conversions;
import market.SICustomer;
import market.swing.CmpNumbers;
import market.swing.ComponentFactory;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import util.swing.AbstractTableEntryDescriptor;
import util.swing.TableEntryDescriptor;
import data.CountingStock;
import data.stdforms.SingleTableFormSheet;
import data.swing.CountingStockTableModel;

/**
 * This FormSheet shows the order of the customer that has to be deduct in a table,
 * it also shows a message-field containing information about removed articles that aren`t avaible yet.
 */
public class FSSellerOrderTable {

    /**
     * @return the order-table.
     *
     * @param shoppingBasket the shopping-basket of the customer that has to be deduct.
     * @param sic the SICustomer in the till-queue related to the customer.
     */
    public static SingleTableFormSheet getOrderTable(CountingStock shoppingBasket, final SICustomer sic){

        final SingleTableFormSheet stfs = SingleTableFormSheet.create(
                "Kundenauftrag",
                shoppingBasket,
                null,
                getTED());

        stfs.addContentCreator(new FormSheetContentCreator(){
            protected void createFormSheetContent(FormSheet fs) {
                JPanel jp_main = new JPanel();
                JPanel jp_upper = new JPanel();
                JComponent jp_table = new JPanel();
                JPanel jp_message = new JPanel();
                JScrollPane jsc = new JScrollPane();
                final JTextArea jta = new JTextArea(10, 200);
                String missingA = sic.getMissingArticles();
                if(missingA.length()>1) jta.setText(getMessage(missingA));
                GridBagConstraints c = new GridBagConstraints();
                GridBagLayout gridbag = new GridBagLayout();

                stfs.getTable().getModel().addTableModelListener(new TableModelListener(){
                    public void tableChanged(TableModelEvent e) {
                        String s = sic.getMissingArticles();
                        if(s.length()>1) jta.setText(getMessage(s));
                    }
                });

                jp_table = fs.getComponent();
                jp_upper.setLayout(gridbag);
                      c.gridx = 0;
                      c.weightx = 1;
                      c.weighty = 1;
                      c.gridy = 0;
                      c.fill = GridBagConstraints.BOTH;
                    gridbag.setConstraints(jp_table, c);
                jp_upper.add(jp_table);
                jp_message.setBorder(ComponentFactory.createInsetBorder("Benachrichtigung"));
                jp_message.setLayout(new BoxLayout(jp_message, BoxLayout.X_AXIS));
                jp_message.add(jsc);
                jsc.setViewportView(jta);

                jp_main.setLayout(new BoxLayout(jp_main, BoxLayout.Y_AXIS));
                jp_main.add(jp_upper);
                jp_main.add(Box.createVerticalStrut(10));
                jp_main.add(jp_message);
                fs.setComponent(jp_main);

                fs.removeAllButtons();
                fs.addButton("Abrechnen",ButtonIDs.BTN_ACCEPT , null);
                fs.addButton("Zurck",ButtonIDs.BTN_BACK , null);
            }
        });
        return stfs;
    }

    /**
     * @return the message to be shown if articles are missing.
     *
     * @param missingA String containing counts and articles that are missed
     */
    private static String getMessage(String missingA){
        return new String("Aufgrund eines vorbergehenden Engpasses\n"+
                    "mussten wir folgende Artikel aus ihrer Bestellung entfernen:"+missingA);
    }

    /**
     * @return the TableEntryDescriptor of the table.
     */
    private static TableEntryDescriptor getTED(){
        AbstractTableEntryDescriptor ted = new AbstractTableEntryDescriptor(){
            public int getColumnCount() {
                return 3;
            }

            public String getColumnName(int nIdx) {
                return (new String[]{ "Artikel", "Preis", "Anzahl"}) [nIdx];
            }

            public Class getColumnClass(int nIdx) {
                return (new Class[] {String.class, Number.class, Number.class}) [nIdx];
            }

            public Object getValueAt(Object oRecord, int nIdx) {
                CIArticle article = Conversions.recordToCIArticle(oRecord);
                int count = ((CountingStockTableModel.Record)oRecord).getCount();
                switch (nIdx) {
                    case 0:
                        return article.getArticleName();
                    case 1:
                        return Conversions.valueToCurrency(
                                CIArticle.getCatalogItemValue().getValue(article));
                    case 2:
                        return new Integer(count).toString();
                }
                return null;
            }

            public boolean canSortByColumn(int nIndex) {
                return true;
             }

             public Comparator getColumnOrder(int nIndex) {
                switch (nIndex) {
                   case 1: return new CmpNumbers(CmpNumbers.BID);
                   case 2: return new CmpNumbers(CmpNumbers.COUNT);
                   default: return null;
                }
             }
        };
        return ted;
    }
}
