package market.stdform;

import java.awt.GridLayout;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import market.CIArticle;
import market.VCPositiveInteger;
import market.swing.ComponentFactory;
import market.swing.JTFCheckable;
import sale.FormSheet;
import sale.FormSheetContentCreator;

/**
 * This FormSheet displays the count of an article in the database and
 * has a checkable textfield were the real count can be filled in.
 */
public class FSWorkerEdit extends FormSheet{

    public static final int JTFC_REAL = 0;

    private CIArticle ci_article;
    private int database;

    /**
     * @param article the CIArticle which`s count is incorrect.
     * @param db the count of this article in the database.
     */
    public FSWorkerEdit(CIArticle article, int db) {
        super("Fehlbestand", null);
        ci_article = article;
        database = db;
        FormSheetContentCreator fscc = new FormSheetContentCreator(){
            protected void createFormSheetContent(FormSheet fs) {
                JPanel jp_main = new JPanel();
                JPanel jp_grid = new JPanel();
                jp_main.setLayout(new BoxLayout(jp_main, BoxLayout.Y_AXIS));
                jp_grid.setLayout(new GridLayout(2,2));

                JTextField jt_database = new JTextField(5);
                jt_database.setText(String.valueOf(database));
                jt_database.setEditable(false);
                JTFCheckable jtfc_real = new JTFCheckable(JTFC_REAL, new VCPositiveInteger("realer Bestand"), 5);

                jp_grid.setBorder(ComponentFactory.createInsetBorder("Artikel: "+ci_article.getArticleName()));
                jp_grid.add(new JLabel("Datenbank-Bestand"));
                jp_grid.add(jt_database);
                jp_grid.add(new JLabel("realer Bestand"));
                jp_grid.add(jtfc_real);

                jp_main.add(Box.createVerticalStrut(160));
                jp_main.add(jp_grid);
                jp_main.add(Box.createVerticalStrut(160));

                fs.setComponent(jp_main);
                fs.removeAllButtons();
                fs.addButton("Korrigiere", ButtonIDs.BTN_OK, null);
                fs.addButton("Zurck", ButtonIDs.BTN_BACK, null);
            }
        };
        this.addContentCreator(fscc);
    }

    /**
     * @return a new {@link FSCheckable} that takes a FSWorkerEdit as argument
     *
     * @param article the CIArticle which`s count is incorrect.
     * @param database the count of this article in the database.
     */
    public static FSCheckable create(CIArticle article, int database){
        return new FSCheckable(new FSWorkerEdit(article, database));
    }
}
