package market.stdform;

import java.util.Hashtable;

import market.CIArticle;
import market.Conversions;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import util.swing.AbstractTableEntryDescriptor;
import data.CountingStock;
import data.stdforms.SingleTableFormSheet;
import data.swing.CountingStockTableModel;

/**
 * This FormSheet displays the order of a customer in a table
 * with one further column to tick off the articles, it is used by workers to execute the order.
 */
public class FSWorkerOrder{

    /**
     * @param customer the customer who`s order gets executed.
     * @param order the order that gets executed.
     * @param done Hashtable with boolean values used for the tick-off column.
     */
    public static SingleTableFormSheet getOrderTable(String customer, CountingStock order, final Hashtable done){

        AbstractTableEntryDescriptor ted = new AbstractTableEntryDescriptor(){
            public int getColumnCount() {
                return 3;
            }

            public String getColumnName(int nIdx) {
                return (new String[]{ "Artikel", "Anzahl", "Erledigt"}) [nIdx];
            }

            public Class getColumnClass(int nIdx) {
                return (new Class[] {String.class, Number.class, Boolean.class}) [nIdx];
            }

            public Object getValueAt(Object oRecord, int nIdx) {
                CIArticle article = Conversions.recordToCIArticle(oRecord);
                int count = ((CountingStockTableModel.Record)oRecord).getCount();
                switch (nIdx) {
                    case 0: return article.getArticleName();
                    case 1: return new Integer(count).toString();
                    case 2: return done.get(article);
                }
                return null;
            }

            public boolean canSortByColumn(int nIndex) {
                return false;
             }
            
            public boolean isElementEditable(Object oRecord, int nIdx) {
            	return nIdx == 2;
            }
            				
            public void setValueAt(Object oRecord, int nIndex, Object oValue) {
            	if (nIndex == 2) {
            		done.put(Conversions.recordToCIArticle(oRecord), oValue);
            	}
            }
        };

        final SingleTableFormSheet stfs = SingleTableFormSheet.create(
                "Kundenauftrag von "+ customer,
                order,
                null,
                ted);

        stfs.addContentCreator(new FormSheetContentCreator() {
            protected void createFormSheetContent(final FormSheet fs) {
                fs.removeAllButtons();
                fs.addButton("Auftrag abschliessen", ButtonIDs.BTN_ACCEPT, null);
                fs.addButton("Fehlbestand", ButtonIDs.BTN_EDIT, null);
                fs.addButton("Zurck", ButtonIDs.BTN_BACK, null);
            }
        });
        return stfs;
    }
}
