package market.swing;

import sale.SaleProcess;
import sale.SalesPoint;
import data.Catalog;
import data.CatalogItem;
import data.CountingStock;
import data.DataBasket;
import data.NotEnoughElementsException;
import data.stdforms.twotableformsheet.CCSStrategy;

/**
 * Defines the behaviour of the shifting buttons in the manager's Purchase-TwoTableFormSheet
 */
public class CCSStrategyMarket extends CCSStrategy {

    public CCSStrategyMarket() {
        super();
    }

    /**
     * Removes at most as much items as are left in the stock.
     * @param p the current SaleProcess.
     * @param sp the SalesPoint on which the TTFS is displayed.
     * @param cSource the source Catalog.
     * @param csDest the destination CountingStock.
     * @param db the transaction's DataBasket.
     * @param ci the affected CatalogItem
     * @param nCount the amount of Items to be shifted.
     */
    protected void moveToSource (SaleProcess p, SalesPoint sp, Catalog cSource,
            CountingStock csDest, DataBasket db, CatalogItem ci, int nCount) {
        int count = csDest.countItems(ci.getName(),db);
        count = Math.min(count, nCount);
        try {
            csDest.remove (ci.getName(), count, db);
        }
        catch (NotEnoughElementsException nee) {
            error (p, NOT_ENOUGH_ELEMENTS_ERROR);
        }
        catch (data.events.VetoException ve) {
            error (p, REMOVE_VETO_EXCEPTION);
        }
        catch (Exception e) {
            System.out.println(e);
        }
    }
}