package market.swing;

import java.io.Serializable;
import java.util.Comparator;

import market.UCustomer;
import market.statistics.Statistics;

/**
 * Compares {@link market.UCustomer UCustomers} by special criteria.
 */
public class CmpCustomersNumbers implements Comparator, Serializable {

    public static final int DISCOUNT = 0;
    public static final int SALES_VOLUME = 1;

    private int option;

    /**
     * @param option <ul><li>0: Users are compared by their discount</li>
     *                  <li>1: Users are compared by their volume of sales</li></ul>
     */
    public CmpCustomersNumbers(int option) {
        this.option = option;
    }

    /**
     * The actual comparison.
     * @param o1 the first customer.
     * @param o2 the second customer.
     * @return an int representing the result of the comparison.
     */
    public int compare(Object o1, Object o2) {
        UCustomer u1 = (UCustomer)o1;
        UCustomer u2 = (UCustomer)o2;
        if (option == 0) {
            return new Double(u1.getDiscount()).intValue() - new Double(u2.getDiscount()).intValue();
        }
        if (option == 1) {
            int i1 = Statistics.getAllowableCustomerRevenue(u1);
            int i2 = Statistics.getAllowableCustomerRevenue(u2);
            return i1 - i2;
        }
        return 0;
    }
}
