package market.swing;

import java.io.Serializable;
import java.util.Comparator;

import market.CIArticle;
import market.Conversions;
import data.swing.CountingStockTableModel;

/**
 * Compares {@link market.CIArticle CIArticles} by numbers.
 */
public class CmpNumbers implements Comparator, Serializable {

    public static final int BID = 0;
    public static final int OFFER = 1;
    public static final int COUNT = 2;
    public static final boolean SUMMED = true;
    public static final boolean SINGLE = false;

    private int chosen;
    private boolean summed;

    /**
     * @param chosen <ul><li>0: CIArticles are compared by their bid</li>
     *                  <li>1: CIArticles are compared by their offer</li>
     *                  <li>2: CIArticles are compared by their amount</li></ul>
     */

    public CmpNumbers(int chosen) {
        this.chosen = chosen;
        this.summed = false;
    }

    /**
     * @param chosen <ul><li>0: CIArticles are compared by their bid</li>
     *                  <li>1: CIArticles are compared by their offer</li>
     *                  <li>2: CIArticles are compared by their amount</li></ul>
     *
     * @param summed Tells the comparator if value to be sorted has been multiplied with the amount
     * of items of a CountingStock or not. This depends, wether the corresponding table shows just the
     * single price of an Article or the summed up prize of a whole bunch of Articles of a kind.
     * This parameter only takes effect for CountingStockRecords
     */
    public CmpNumbers(int chosen, boolean summed) {
        this.chosen = chosen;
        this.summed = summed;
    }

    /**
     * The actual comparison.
     * @param o1 the first CIArticle.
     * @param o2 the second CIArticle.
     * @return an int representing the result of the comparison.
     */
    public int compare(Object o1, Object o2) {
        CIArticle c1 = Conversions.recordToCIArticle(o1);
        CIArticle c2 = Conversions.recordToCIArticle(o2);
        String name1 = c1.getName();
        String name2 = c2.getName();
        int count1 = 1;
        int count2 = 1;
        int amount1 = 1;
        int amount2 = 1;
        //CountingStockTableObjects have extra value count, so evaluate them first
        if (o1 instanceof CountingStockTableModel.Record) {
           count1 = ((CountingStockTableModel.Record)o1).getCount();
           count2 = ((CountingStockTableModel.Record)o2).getCount();
        }
        if (summed) {
            amount1 = count1;
            amount2 = count2;
        }
        switch (chosen) {
           case BID:   return amount1 * c1.getBid() - amount2 * c2.getBid();
           case OFFER: return amount1 * c1.getOffer() - amount2 * c2.getOffer();
           case COUNT: return count1 - count2;
        }
        return 0;
    }
}