package market.swing;

import java.util.List;

import javax.swing.table.AbstractTableModel;

import market.Conversions;
import market.statistics.HistoryEntry;

/**
 * TableModel for tables that display {@link HistoryEntry HistoryEntries}.
 */
public class HistoryEntryModel extends AbstractTableModel {

    public final static int PRICE_HISTORY = 0;
    public final static int ORDER_HISTORY = 1;

    private List data;
    private int type;

    /**
     * @param data the history to be displayed
     * @param type the kind of history:
     *      <ul><li>0: interpreted as {@link market.statistics.CISalesStats#priceHistory}</li>
     *          <li>1: interpreted as {@link market.statistics.CISalesStats#orderHistory}</li></ul>
     */
    public HistoryEntryModel(List data, int type) {
        super();
        this.data = data;
        this.type = type;
    }

    /**
     * @param i the affected column.
     * @return columns' names.
     */
    public String getColumnName(int i) {
        return new String[] {"Datum", type == 0 ? "Preis" : "Bestellt"}[i];
    }

    /**
     * @return the number of the table's rows.
     */
    public int getRowCount() {
        return data.size();
    }

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 2;
    }

    /**
     * @param column the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass(int column) {
        return String.class;
    }

    /**
     * @param row the affected row.
     * @param column the affected column.
     * @return the table cell's value.
     */
    public Object getValueAt(int row, int column) {
        HistoryEntry oRecord = (HistoryEntry)data.get(row);
        switch (column) {
            case 0:
                return oRecord.getDate();
            case 1:
                return type == 0 ? Conversions.doubleToCurrency(oRecord.getValue()) :
                        Integer.toString(oRecord.getValue());
            default:
                return "";
        }
    }
}